model = {
	// definitions
	_def: {
		itemsPerPage: 40,
		page: 1,
		slotTolerance: 50,
		slotDefaultHeight: 200,
		scrollBottomTolerance: 3000,
		filter: {
			maxPrice: null,
			searchQuery: null,
			color: [],
			inSale: null
		},
		title: ''
	},
	onLoad: function(sC,eC){
		var self = this;

		app.nav.showLevel(2);

		self._def.filter.searchQuery = self.params.get("searchQuery");

		// prepare scroll-bottom-distance
		self.func.set("scroll-bottom-tolerance",self._def.scrollBottomTolerance);

		// set observableData
		self.data.inspireSourcesColumnLeft = ko.observableArray();
		self.data.inspireSourcesColumnRight = ko.observableArray();

		// this is the refresher var for all DOM events
		self.computedRefresher = ko.observable(0);

		// the temporary stored products from the api
		self.tempProductList = [];

		// this is a var to stop bottomScroll loading (used, if no more products available)
		self.productLoadStatus = true;

		// this is a var, to prevent double-bottom(in/out) scroll and load data multiple times
		self.productLoadHold = false;

		// this is the computed function for all new elements
		self.computedProductList = ko.computed(function(){
			var _x = self.computedRefresher();

			var _tmp = self.getSmallerColumn();

			var cPos = _tmp.start;
			var startCount = 0;
			var startAmount = _tmp.amount;
			var startUse = _tmp.use;
			_(self.tempProductList).forEach(function(preProduct){
				if (startCount < startAmount){
					if (startUse == 'left'){
						self.data.inspireSourcesColumnLeft.push(preProduct);
					}else{
						self.data.inspireSourcesColumnRight.push(preProduct);
					}
					startCount++;
					return;
				}

				if (cPos%2==0){
					self.data.inspireSourcesColumnLeft.push(preProduct);
				}else{
					self.data.inspireSourcesColumnRight.push(preProduct);
				}
				cPos++;
			});
			// clean product list
			self.tempProductList = [];

			// hide-show noResults container
			if (self.productLoadStatus == false){
				if (self.data.inspireSourcesColumnLeft().length == 0 && self.data.inspireSourcesColumnRight().length == 0){
					self.page.getNoResults().removeClass("hidden");
				}

				// hide loadingContainer
				self.page.getLoading().addClass("hidden");
			}else{
				// hide no results container
				if(!self.page.getNoResults().hasClass("hidden")){
					self.page.getNoResults().addClass("hidden");
				}

				// show loading container
				/*if(self.page.getLoading().hasClass("center")){
					self.page.getLoading().removeClass("center");
				}*/
				self.page.getLoading().removeClass("hidden");
			}
		});

		// slot position-data || {domPos: 1234, status: false, id: null}
		self.slotData = {};

		var _loadAPI = function(){
			app.nav.showContainer("loading",true);

			// get products from API
			self.getDataFromAPI(
				function(iData){
					self._def.title = iData.name;

					// refresh list now
					self.computedRefresher(self.computedRefresher()+1);

					// workaround for iOS devices
					self.loadAllDataSlots();
					sC();
				},
				function(result){
					if(result.result===false || result.statusText=="timeout"){
						app.nav.alert.show($.t('searchBrowse.messages.errorUnknownError'),'error');
					}
					else{
						app.nav.alert.show($.t('searchBrowse.messages.noProductsFound'),'info');
					}
					self.page.remove();
					eC();
				}
			);
		};

		_loadAPI();
	},
	registerScrollPositions: function(){
		var self = this;
		var cPage = self.page.get();

		_.forEach(self.slotData,function(cSlot){
			if (_.isNull(cSlot.domPos)){
				var dSlot = cPage.find("[data-slot='"+cSlot.id+"']");
				if (dSlot.length > 0){
					self.slotData[cSlot.id].domPos = dSlot.offset().top;
				}
			}
		});
	},
	loadDataSlot: function(slotId){
		var self = this;
		var cPage = self.page.get();
		var dImg = cPage.find("[data-img='"+slotId+"']");
		var slotData = self.slotData[slotId];
		if (dImg.length > 0 && slotData.img != ""){
			dImg.attr("src",slotData.img);
			delete self.slotData[slotId];
		}
	},
	onFocus: function(sC,eC){
		var self = this;

		app.nav.showLevel(2);

		var _toggleContainer = function(){
			self.toggleFilterContainer(true);
			models('#filter').toggleSearch(true, true);
			models('#filter').setColorsAndSale(true);
			self.refreshFilter();
			sC();
		};

		if (!app.nav.page.is.loadedPage("#filter")){
			app.nav.loadPage("#filter",{pageLvl: 3, noHistory: true, restoreModel: true},_toggleContainer);
		}else{
			_toggleContainer();
		}
	},
	onBlur: function(sC,eC){
		var self = this;
		self.toggleFilterContainer(false);
		sC();
	},
	afterShow: function(sC,eC){
		var self = this;
		self.loadAllDataSlots();
		sC();
		return;
		self.registerScrollPositions();
		// fire onScroll for first elements
		self.onScroll(null,null,self.page.getScrollData());
		sC();
	},
	loadAllDataSlots: function(){
		var self = this;
		_.forEach(self.slotData,function(cSlot,slotId){
			self.loadDataSlot(slotId);
		});
	},
	onFilterRefresh: function(newFilterValues){
		var self = window.model;
		self._def.filter.maxPrice = newFilterValues.maxPrice;
		self._def.filter.searchQuery = newFilterValues.searchQuery;
		self._def.filter.color = newFilterValues.color;
		self._def.filter.inSale = newFilterValues.inSale;

		self.productLoadStatus = true;

		// append center class for big scrolling
		//self.page.getLoading().addClass("center");

		// clear old data
		self.data.inspireSourcesColumnLeft.removeAll();
		self.data.inspireSourcesColumnRight.removeAll();

		// reset onload transitions
		self.func.get("img-onload-transition").cpos = 0;

		// reset page
		self._def.page = 1;

		// call 'scrollBottom' to reload
		self.onScrollBottom();
	},
	onScroll: function(sC,eC,scrollData){
		((_.isNull(sC))?sC=function(){}:null);
		((_.isNull(eC))?eC=function(){}:null);
		var self = this;

		sC();
		return;

		var cViewport = scrollData.viewport;
		var cPage = self.page.get();

		_.forEach(self.slotData,function(cSlot,slotId){
			if ((cSlot.domPos-scrollData.top-self._def.slotTolerance) <= cViewport){
				self.loadDataSlot(slotId);
			}
		});
		sC();
	},
	onScrollBottom: function(sC,eC,sData){
		((!sC)?sC=function(){}:null);
		((!eC)?eC=function(){}:null);
		((!sData)?sData={}:null);
		var self = this;

		// check if more products are empty
		if (!self.productLoadStatus){
			sC();
			return;
		}

		// check pHold
		if (self.productLoadHold){
			sC();
			return;
		}
		// set pHold
		self.productLoadHold = true;

		// show loadingContainer
		self.page.getLoading().removeClass("hidden");

		var _loadAPI = function(){
			// get products from API
			self.getDataFromAPI(
				function(iData){
					// refresh list now
					self.computedRefresher(self.computedRefresher()+1);

					// workaround for iOS devices
					self.loadAllDataSlots();
					self.productLoadHold = false;
					self.func.set("scroll-bottom-state",true);
					sC();
				},
				function(result){
					if(result.result===false || result.statusText=="timeout"){
						app.nav.alert.show($.t('searchBrowse.messages.errorUnknownError'),'error');
					}
					// workaround for iOS devices
					self.loadAllDataSlots();
					self.productLoadHold = false;
					self.func.set("scroll-bottom-state",true);
					sC();
				}
			);
		};

		window.setTimeout(function(){
			_loadAPI();
		},250);
	},
	refreshFilter: function(){
		var self = this;
		var cValues = {};
		var filterModel = models("#filter");
		if(self._def.filter.maxPrice != null){
			cValues.maxPrice = self._def.filter.maxPrice;
			if(app.settings.get('CURRENCY').toLowerCase() == 'gbp'){
				cValues.maxPrice = Math.ceil(cValues.maxPrice * app.settings.get('CURRENCY_GBP'));
			}
			else if(app.settings.get('CURRENCY').toLowerCase() == 'pln'){
				cValues.maxPrice = Math.ceil(cValues.maxPrice * app.settings.get('CURRENCY_PLN'));
			}
		}
		else{
			cValues.maxPrice = filterModel._def.filter.maxPriceDefault;
			if(app.settings.get('CURRENCY').toLowerCase() == 'gbp'){
				cValues.maxPrice = Math.ceil(cValues.maxPrice * app.settings.get('CURRENCY_GBP'));
			}
			else if(app.settings.get('CURRENCY').toLowerCase() == 'pln'){
				cValues.maxPrice = Math.ceil(cValues.maxPrice * app.settings.get('CURRENCY_PLN'));
			}
		}
		if(self._def.filter.color.length > 0){
			cValues.color = self._def.filter.color;
		}
		else{
			cValues.color = filterModel._def.filter.colorDefault;
		}
		if(self._def.filter.inSale !== null){
			cValues.inSale = self._def.filter.inSale;
		}
		else{
			cValues.inSale = filterModel._def.filter.inSaleDefault;
		}
		if(self._def.filter.searchQuery !== null){
			cValues.searchQuery = self._def.filter.searchQuery;
		}
		else{
			cValues.searchQuery = filterModel._def.filter.searchQueryDefault;
		}
		models("#filter").registerCallback(self.onFilterRefresh);
		models("#filter").setValues(cValues);
	},
	toggleFilterContainer: function(showFilter){
		var self = this;

		if (showFilter){
			app.nav.showLevel(3);
		}else{
			app.nav.hideLevel(3);
		}
	},
	getDataFromAPI: function(sC,eC){
		((!eC)?eC=function(){}:null);
		var self = this;

		var rParams = {
			product_search:{
				'page': self._def.page,
				'per_page': self._def.itemsPerPage,
				'in_sale': self._def.filter.inSale,
				'q': self._def.filter.searchQuery
			}
		};

		if (self._def.filter.maxPrice != null){
			rParams.product_search.price = {
				max : self._def.filter.maxPrice
			};
		}
		if (self._def.filter.color.length > 0){
			rParams.product_search.color = self._def.filter.color;
		}

		app.api.doAPIRequestAsync(
			':apiPost',
			'product_searches',
			rParams,
			'post',
			function(result){
				if(result.result===true){
					var iData = result.data;
					// check for valid result
					if (iData){
						self._def.page++;

						if (iData.products && iData.products.length > 0){
							_(iData.products).forEach(function(cProduct){
								var pProduct = self.prepareProduct(cProduct);
								self.tempProductList.push(pProduct);
								self.slotData[cProduct.id] = {domPos: null,id:pProduct.id,img: pProduct.image};
							});

							// check for max length reached
							if (iData.products.length < self._def.itemsPerPage){
								self.productLoadStatus = false;
							}
						}else{
							// prevent loading additional data
							self.productLoadStatus = false;
						}
						sC(iData);
					}else{
						eC(result);
					}
				}
				else{
					eC(result);
				}
			},
			eC
		);
	},
	getSmallerColumn: function(){
		var self = this;
		var cPage = self.page.get();
		var leftColumnSize = cPage.find(".col-50:first").height();
		var rightColumnSize = cPage.find(".col-50:last").height();

		var useAmount = 0;
		var use = "left";
		var startPos = 0;

		if (leftColumnSize == rightColumnSize){
			// do nothing
		}else if(leftColumnSize > rightColumnSize){
			useAmount = Math.ceil((leftColumnSize-rightColumnSize)/self._def.slotDefaultHeight);
			use = "right";
			startPos = 0;
		}else{
			useAmount = Math.ceil((rightColumnSize-leftColumnSize)/self._def.slotDefaultHeight);
			startPos = 1;
		}
		return {'use': use, amount: useAmount, start: startPos};
	},
	prepareProduct: function(cProduct){
		var self = this;

		var cDataContainer = {
			id: cProduct.id,
			title: cProduct.title,
			badge: cProduct.badge,
			image: 'http:'+cProduct.default_image.full,
			sale_percentage: ((cProduct.sale_percentage!=null)?cProduct.sale_percentage:"")
		};

		if(app.system.checkProductOnPinnedProductsList(cProduct.id)){
			cDataContainer.pinned = true;
		}
		else{
			cDataContainer.pinned = false;
		}
		if(cProduct.sold_in_unit==false && cProduct.discounted==false){
			cDataContainer.price = app.system.formatAmount(cProduct.price.cents, cProduct.price.currency);
			cDataContainer.price_small = ' ';
			cDataContainer.price_sale = false;
			cDataContainer.price_small_sale = false;
		}
		else if(cProduct.sold_in_unit==false && cProduct.discounted){
			cDataContainer.price = app.system.formatAmount(cProduct.price.cents, cProduct.price.currency);
			cDataContainer.price_small = app.system.formatAmount(cProduct.undiscounted_price.cents, cProduct.undiscounted_price.currency);
			cDataContainer.price_sale = true;
			cDataContainer.price_small_sale = true;
		}
		else if(cProduct.sold_in_unit && cProduct.discounted==false){
			cDataContainer.price = app.system.formatAmount(cProduct.price.cents, cProduct.price.currency);
			cDataContainer.price_small = app.system.formatAmount(cProduct.base_price.cents, cProduct.base_price.currency) + '/' + $.t('pdp.units.'+cProduct.unit);
			cDataContainer.price_sale = false;
			cDataContainer.price_small_sale = false;
		}
		else if(cProduct.sold_in_unit && cProduct.discounted){
			cDataContainer.price = app.system.formatAmount(cProduct.price.cents, cProduct.price.currency);
			cDataContainer.price_small = app.system.formatAmount(cProduct.base_price.cents, cProduct.base_price.currency) + '/' + $.t('pdp.units.'+cProduct.unit);
			cDataContainer.price_sale = true;
			cDataContainer.price_small_sale = false;
		}
		return cDataContainer;
	}
};