/********************************************************************
 * app.api															*
 ********************************************************************
 * request / send data from/to the api location						*
 ********************************************************************/
(function(app){
	app.api = {
		// need this for eListener
		_className: 'api',

		// returned errors from the api
		_errors: [],

		// default targets - writeable by settings
		_targets: {
			':manifest': '/manifest',
			':file': '/manifest/file',
			':api': '/api/r',
			':apiPost': '/api/rp',
			':apiDelete': '/api/rd',
			':apiLoginFB': '/api/login_fb',
			':apiLoginDW': '/api/login_dw',
			':apiRegisterDW': '/api/register_dw',
			':apiResendEmail': '/api/resend_validation_email',
			':apiPasswordReset': '/api/password_reset',
			':apiGetCurrency': '/api/getcurrency',
			':pnRegister': '/pn/register',
			':pnDelete': '/pn/delete',			
			':support': '/dev/support'
		},

		doRequest: function(rAction,rParams,rMethod){
			((!rParams)?rParams={}:null);
			((!rMethod)?rMethod="get":null);
			var self = this;

			// clear all prev errors
			self.clearErrors();

			var _funcParams = {
				async: false,
				apiAction: rAction,
				apiParams: rParams,
				apiMethod: rMethod
			};

			var returnData = null;

			self._rFunc(_funcParams,function(apiRes){
				var dVal = self._rDataValidator(apiRes);
				if (dVal.result){
					returnData = dVal.data;
				}
			});
			return returnData;
		},
		doRequestAsync: function(rAction,rParams,rMethod,successCallback,errorCallback,progressCallback){
			((!rParams)?rParams={}:null);
			((!rMethod)?rMethod="get":null);
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			((!progressCallback)?progressCallback=function(p){}:null);
			var self = this;

			// clear all prev errors
			self.clearErrors();

			var _funcParams = {
				async: true,
				apiAction: rAction,
				apiParams: rParams,
				apiMethod: rMethod
			};

			self._rFunc(
				_funcParams,
				function(apiRes){
					var dVal = self._rDataValidator(apiRes);
					if (dVal.result){
						if (!_.isUndefined(dVal.auth)){
							_.forEach(dVal.auth,function(aValue,aKey){
								var tmpVal = app.settings.get('api.'+aKey);
								if (tmpVal != aValue){
									app.settings.set('api.'+aKey,aValue,true);
								}
							});
						}
						successCallback(dVal.data);
					}else{
						errorCallback(dVal.data);
					}
				},
				errorCallback,
				progressCallback
			);
			return null;
		},
		doAPIRequestAsync: function(aAction,rAction,rParams,rMethod,successCallback,errorCallback){
			((!rParams)?rParams={}:null);
			((!rParams.locale)?rParams.locale=app.settings.get('LANGUAGE'):null);
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			((!rMethod)?rMethod="get":null);
			
			var self = this;

			// clear all prev errors
			self.clearErrors();

			var _funcParams = {
				async: true,
				apiAction: aAction,
				apiParams: {'request': rAction, 'rparams': rParams, 'auth': {'sessionToken': app.settings.get('api.sessionToken'),'rememberToken': app.settings.get('api.rememberToken')}},
				apiMethod: rMethod
			};

			self._rFunc(
				_funcParams,
				function(apiRes){
					var dVal = self._rDataValidator(apiRes);
					if (dVal.result){
						if (!_.isUndefined(dVal.auth)){
							_.forEach(dVal.auth,function(aValue,aKey){
								var tmpVal = app.settings.get('api.'+aKey);
								if (tmpVal != aValue){
									app.settings.set('api.'+aKey,aValue,true);
								}
							});
						}
						successCallback(dVal.data);
					}else{
						errorCallback(dVal.data);
					}
				},
				errorCallback
			);
			return null;
		},
		doRequestBinary: function(rAction,rParams,rMethod){
			((!rParams)?rParams={}:null);
			((!rMethod)?rMethod="get":null);
			var self = this;

			var _funcParams = {
				async: false,
				apiAction: rAction,
				apiParams: rParams,
				apiMethod: rMethod
			};

			var returnData = null;

			self._bFunc(
				_funcParams,
				function(apiRes){
					returnData = apiRes;
				}
			);
			return returnData;
		},
		doRequestBinaryAsync: function(rAction,rParams,successCallback,errorCallback,progressCallback){
			((!rParams)?rParams={}:null);
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			((!progressCallback)?progressCallback=function(p){}:null);
			var self = this;

			var _funcParams = {
				async: true,
				apiAction: rAction,
				apiParams: rParams,
				apiMethod: 'get'
			};

			self._bFunc(
				_funcParams,
				successCallback,
				errorCallback,
				progressCallback
			);
			return null;
		},
		_rDataValidator: function(inputData){
			var self = this;
			var cData = inputData;
			if (!_.isObject(cData)){
				cData = parse(cData);
			}

			if (cData.result){
				var retData = {
					result: true,
					data: cData.data
				};
				if (!_.isUndefined(cData.auth)){
					retData['auth'] = cData.auth;
				}
				return retData;
			}else{
				// request had errors
	    		_.forEach(cData.errors,function(eObj){
	    			self._setError(eObj.id,eObj.message);
	    		});

				return {
					result: false,
					data: cData.errors
				};
			}
		},
		// central binary file func
		_bFunc: function(rData,successCallback,errorCallback,progressCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			((!progressCallback)?progressCallback=function(p){}:null);
			var self = this;

			// verify is ready
        	if (!_.isReady(self)){
        		errorCallback("api.notReady");
        		return false;
        	}

        	// check for targetActions
        	if(rData.apiAction.substr(0,1) == ":"){
        		if (!_.isUndefined(self._targets[rData.apiAction])){
        			rData.apiAction = self._targets[rData.apiAction];
        		}else{
        			rData.apiAction = "";
        		}
        	}

			// validate action
			if (_.isEmpty(rData.apiAction)){
				errorCallback("api.emptyAction");
				return false;
			}else if(rData.apiAction.substr(0,1) != "/"){
				rData.apiAction = "/"+rData.apiAction;
			}				

			// get current API-URL from settings
			var API_URL = app.settings.get("api.host");

			// the currentScope
			var _REQSCOPE = {
				platform: app.settings.get("PLATFORM"),
				platform_version: app.settings.get("PLATFORM_VERSION"),
				rev: app.settings.get("REV"),
				key: app.settings.get("api.sharedSecret"),
				timestamp: self._getTimestamp(),
				method: rData.apiMethod,
				mstatus: app.settings.get("api.mstatus"),
				mid: app.settings.get("api.mid")
			};
			var SCOPE = _.assign(rData.apiParams,{_req: _REQSCOPE});

			// send over
			console.log("api.bRequest: SEND ("+API_URL+rData.apiAction+")",SCOPE);

			// generate SCOPE-PARAMS (GET only)
			var gParams = "?"+uriConv(SCOPE);

			var xhr = new XMLHttpRequest();
			xhr.open('GET', API_URL+rData.apiAction+gParams, true);
			xhr.responseType = 'blob';

			xhr.onload = function(e){
				if (this.status == 200){
					console.log("api.bRequest: RECEIVE ("+rData.apiAction+gParams+")",xhr.response);
					var blob = this.response;
					successCallback(blob);
				}else if (this.status == "404"){
					console.error("api.bRequest: ERROR ("+rData.apiAction+gParams+")",xhr.response);
					successCallback(null);
				}
			};

			xhr.onerror = function(e) {
                errorCallback(e);
            };

			xhr.onreadystatechange = function(){
				if (xhr.readyState==4 && xhr.status==404){
					console.error("api.bRequest: ERRORSTATE ("+rData.apiAction+gParams+")",xhr.response);
					successCallback(null);
				}
			};

			//Download progress
			xhr.addEventListener("progress", function(evt){
				if (evt.lengthComputable) {
					var percentComplete = evt.loaded / evt.total;
					console.info("api.bProgress: "+percentComplete);
					progressCallback(percentComplete);
				}
			}, false);


			xhr.send();
			return null;
		},
		// central request func
		_rFunc: function(rData,successCallback,errorCallback,progressCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			((!progressCallback)?progressCallback=function(p){}:null);
			var self = this;

			// verify is ready
        	if (!_.isReady(self)){
        		errorCallback("api.notReady");
        		return false;
        	}

        	// check for targetActions
        	if(rData.apiAction.substr(0,1) == ":"){
        		if (!_.isUndefined(self._targets[rData.apiAction])){
        			rData.apiAction = self._targets[rData.apiAction];
        		}else{
        			rData.apiAction = "";
        		}
        	}

			// validate action
			if (_.isEmpty(rData.apiAction)){
				errorCallback("api.emptyAction");
				return false;
			}else if(rData.apiAction.substr(0,1) != "/"){
				rData.apiAction = "/"+rData.apiAction;
			}				

			// get current API-URL from settings
			var API_URL = app.settings.get("api.host");

			// the currentScope
			var _REQSCOPE = {
				platform: app.settings.get("PLATFORM"),
				platform_version: app.settings.get("PLATFORM_VERSION"),
				rev: app.settings.get("REV"),
				key: app.settings.get("api.sharedSecret"),
				timestamp: self._getTimestamp(),
				method: rData.apiMethod,
				mstatus: app.settings.get("api.mstatus"),
				mid: app.settings.get("api.mid")
			};
			var SCOPE = _.assign(rData.apiParams,{_req: _REQSCOPE});

			// send over
			console.log("api.rRequest: SEND ("+rData.apiAction+")",SCOPE);

			var blob = null;

			$.ajax({
				xhr: function(){
					var xhr = new window.XMLHttpRequest();
					

					//Upload progress
					/*xhr.upload.addEventListener("progress", function(evt){
						if (evt.lengthComputable) {
							var percentComplete = evt.loaded / evt.total;
							console.log("U:"+percentComplete);
							progressCallback(percentComplete);
						}
					}, false);*/

					//Download progress
					xhr.addEventListener("progress", function(evt){
						if (evt.lengthComputable) {
							var percentComplete = evt.loaded / evt.total;
							console.info("api.rProgress: "+percentComplete);
							progressCallback(percentComplete);
						}
					}, false);
					return xhr;
				},
    			url: API_URL+rData.apiAction,
    			data: SCOPE,
    			type: 'post',
    			async: rData.async,
    			dataType: "text",
        		cache: false,
    			timeout: app.settings.get("api.timeout"),
    			success: function(data){
    				console.info("api.rRequest: RECEIVE ("+rData.apiAction+")",data);
    				successCallback(data);
    				return;
    			},
    			error: function(jqXHR, textStatus, errorThrown) {
    				console.error("api.rRequest: RECEIVE ("+rData.apiAction+")",jqXHR);
    				errorCallback(jqXHR);
    				return;
    			}
    		});
    		return null;
		},
		getErrors: function(){
			var self = this;
			return self._errors;
		},
		clearErrors: function(){
			var self = this;
			self._errors = [];
		},
		
		/****************
		 * PRIVATE AREA *
		 ****************/
		
		_setError: function(errorId,errorMsg){
			var self = this;
			self._errors.push({id:errorId,message:errorMsg});
		},
		_getTimestamp: function(){
			var unix = Math.round(+new Date()/1000);
			return unix;
		},
		_init: function(){
			var self = this;

			// register READY/init function
			eListener.register("settings",self._initReady);
		},
		_initReady: function(){
			// reference to global - unable to load through eListener (this is eListener)
			var self = app.api;

			// update targets func
			var _updateTargets = function(){
				var nTargets = app.settings.get("api.targets");
				if ((!_.isNull(nTargets)) && (!_.isEmpty(nTargets))){
					self._targets = nTargets;
				}
			};

			// update targets
			_updateTargets();

			// set ready
			_.setReady(self);
		}
	};
	
	app.api._init();
})(window.app);