/********************************************************************
 * app                                                              *
 ********************************************************************
 * this is the main application class                               *
 ********************************************************************/

var app = {
    _data:{
        pushedBackOnPageZero: false,
        pushedBackOnPageZeroTimeout: null,
        storageReadyCalled: false,
        deviceReadyCalled: false,
        appInitCalled: false
    },
    _timer: {},
    _registers: {},
    _i18n: {
        _cLang: null,
        _lang: {
            'de': {
                'deviceOffline':        'GERÄT IST OFFLINE',
                'deviceOfflineMessage': 'Du musst online sein, um diese App zu benutzen.'
            }
        },
        getCurrentLang: function(){
            var self = app._i18n;

            if (self._cLang != null){
                return self._cLang;
            }
            // first init of current Language
            if (navigator && navigator.language){
                self._cLang = navigator.language.split("-")[0];
            }else{
                self._cLang = 'de';
            }

            return self._cLang;
        },
        get: function(key,alt){
            ((!alt)?alt="":null);
            var self = app._i18n;
            var cLang = self.getCurrentLang();

            if ((!_.isUndefined(self._lang[cLang])) && (!_.isUndefined(self._lang[cLang][key]))){
                return self._lang[cLang][key];
            }
            return alt;
        },
        prepare: function(){
            var self = this;
            // replace lang data
            $("[data-i18n-s]").each(function(){
                var obj = $(this);
                var langKey = obj.attr("data-i18n-s");

                var trans = self.get(langKey);
                if (!_.isNull(trans)){
                    obj.html(trans);
                }
            });
            console.info("app.i18n.prepare - done");
        }
    },
    prepareForInit: function(){
        var self = this;
        // sets the initTimer
        self._timer.init = self._getTimestamp();

        // bind all required events
        this.bindEvents();

        // load navigationClass - necassary to show different VIEWS
        loadScript(GLOBALS.includePath+"shared/js/nav.js");

        // startup translations
        this._i18n.prepare();

        // deviceReady fallback
        this._callSecureTimer();
    },
    initialize: function() {
        // load i18n, if required
        if (GLOBALS.i18n.status){
            loadScript(GLOBALS.includePath+"shared/js/thirdparty/i18next-1.7.3.min.js");
        }

        // load application files
        loadScript(GLOBALS.includePath+"shared/js/settings.js");
        loadScript(GLOBALS.includePath+"shared/js/storageHelper.js");
        loadScript(GLOBALS.includePath+"shared/js/navHelper.js");
        loadScript(GLOBALS.includePath+"shared/js/api.js");
        loadScript(GLOBALS.includePath+"shared/js/rus.js");

        // THIS IS NOT A FRAMEWORK-CLASS - use this for all central-functions for your app
        loadScript(GLOBALS.includePath+"shared/js/system.js");
        
        // include additional JS-LIB for iOS devices
        if(app.settings.get("PLATFORM") == "ios"){
            //loadScript(GLOBALS.includePath+"shared/js/thirdparty/iscroll-probe.js");
        }
        // bugfix for issue that android doesn't scroll to active input
        if(app.settings.get("PLATFORM") == 'android'){
            window.addEventListener("resize", function(){
                if(document.activeElement.tagName=="INPUT"){
                    window.setTimeout(function(){
                        document.activeElement.scrollIntoViewIfNeeded();
                    },0);
                }
            });
        }

        // set app.init ready
        app._data.appInitCalled = true;

        // fallback for 'to-early' deviceReady call
        app._callStorageReady();

        console.info("app.initialize: done");
    },
    bindEvents: function() {
        document.addEventListener('deviceready', app.onDeviceReady, false);
        document.addEventListener('pause', app.onDevicePause, false);
        document.addEventListener('resume', app.onDeviceResume, false);
        document.addEventListener('offline', app.onDeviceOffline, false);
        document.addEventListener('online', app.onDeviceOnline, false);
        document.addEventListener("backbutton", app.onDeviceBack, false);

        window.addEventListener('message', app.onMessage, false);
        
        // @deprecated
        //document.addEventListener("touchmove", this.onTouchmove, true);
        //document.addEventListener("scroll", this.onScroll, false);

        // STARTUP BINDINGS
        // register READY
        eListener.register(["rus","navHelper"],function(){
            app.onStartupReady();
        });
    },
    onMessage: function(e){
        // toDo: validate origin
        if (_.isObject(e.data)){
            if (!_.isReady(app.helper.nav)){
                return;
            }
            switch (e.data.type){
                case "window":
                    _(e.data.scopes).forEach(function(cScope){
                        app.helper.nav.window.cb(e.data.id,cScope.action,cScope.data);
                    });                    
                break;
                case "iframe":
                    _(e.data.scopes).forEach(function(cScope){
                        app.helper.nav.iframe.cb(e.data.id,cScope.action,cScope.data);
                    });
                break;
            }
        }
    },
    // @deprecated...
    onTouchmove: function(){
        app._callRegisterFunc("onTouchmove");
    },
    onDeviceBack: function() {
        var self = app;
        var _successCb = function(sCounter){
            if (parseInt(sCounter) > 0){
                return;
            }
            else{
                if(app._data.pushedBackOnPageZero){
                    self.exit();
                }
                else{
                    app._data.pushedBackOnPageZero = true;
                    app.nav.alert.show($.t('general.messages.tabAgainToClose'), 'info');
                    app._data.pushedBackOnPageZeroTimeout = setTimeout(function(){app._data.pushedBackOnPageZero = false;}, 5000);
                }
            }
        };
        app.nav.goBack(_successCb);
    },
    onDeviceReady: function() {
        // check for deviceReady called status
        if (app._data.deviceReadyCalled){
            return false;
        }

        // set deviceReadyStatus
        app._data.deviceReadyCalled = true;

        // call storageReady func
        app._callStorageReady();

        console.info("app.onDeviceReady: done");
    },
    onDeviceOffline: function(){
        app.settings.set("OFFLINE",true);

        // show init container
        app.nav.showContainer("offline",true);
    },
    onDeviceOnline: function(){        
        app.settings.set("OFFLINE",false);
        
        if (!_.isEmpty(app.nav)){
            app.nav.hideContainer("offline");
        }
        // run breakPoint function
        app._callRegisterFunc('breakPoint',true);
    },
    onStartupReady: function(){
        if (!app.settings.get("STARTUP")){
            this._prepareBody();

            // show init container
            app.nav.showContainer("main");

            // loads the startPage, defined in the GLOBALS
            app.nav.loadPage(GLOBALS.startup.pageName,GLOBALS.startup.pageParams);
            app.settings.set("STARTUP",true);
        }
    },
    onDevicePause: function(){
        console.info("app.onDevicePause: done");
    },
    onDeviceResume: function(){
        var self = app;

        // check 'offline' time
        var rTimer = app.settings.get("RESTART_TIMER");
        var cTime = self._getTimestamp();
        if (cTime-rTimer > self._timer.init){
            console.info("app.onDeviceResume: RESTART");
            app.restart();
            return false;
        }

        // check for PUSH - Plugin
        if (!_.isUndefined(PushNotification)){
            PushNotification.getIncoming(function(result){
                if(result.message != null && result.message != ""){
                    app.system.push.handler(result);
                }
            });
        }

        // check data timeouts
        app.system.checkDataTimeouts();

        console.info("app.onDeviceResume: done");
    },
    // restart the app
    restart: function(){
        console.info("app.restart");
        app.nav.hideContainer("error");
        app.nav.showContainer("loading");

        if ((!_.isUndefined(app.nav)) && (!_.isEmpty(app.nav)) && (!_.isUndefined($.t))){
            app.nav.alert.show($.t("general.messages.restartingApp"),"warning");
        }

        var _restartAppFunc = function(){
            // toDo: iOS restart
            if (app.settings.get("PLATFORM") == "android"){
                navigator.app.loadUrl("file:///android_asset/www/index.html", {wait:1000});
            }else{
                document.location.href = 'index.html';
            }
        };
        window.setTimeout(_restartAppFunc,2000);
    },
    // exit this app
    exit: function(){
        console.info("app.exit");
        if(navigator.app){
            navigator.app.exitApp();
        }else if(navigator.device){
            navigator.device.exitApp();
        }
    },
    _prepareBody: function(){
        var cPlatform = app.settings.get("PLATFORM");
        // prepare platform type
        $("body").addClass("device-"+cPlatform);

        // prepare platform version
        if (cPlatform == "android"){
            var tmpArray = navigator.userAgent.split("(");
            var tmpSplit = tmpArray[1].split(" ");
            var pointer = 0;
            _(tmpSplit).forEach(function(cVal,cPos){
                if (cVal == "Android" && pointer == 0){
                    pointer = cPos+1;
                }
            });
            var devVersion = tmpSplit[pointer].replace(/;/g,"").replace(/\./g,"_");
            $("body").addClass("device-version-"+devVersion);
            $("body").addClass("device-type-unknown");
        }else{
            var tmpArray = navigator.userAgent.split("(");
            var tmpSplit = tmpArray[1].split(";");
            var devType = tmpSplit[0].replace(/ /g,"_");
            var osSplit = tmpSplit[1].split(" ");
            var pointer = 0;
            _(osSplit).forEach(function(cVal,cPos){
                if (cVal == "OS" && pointer == 0){
                    pointer = cPos+1;
                }
            });

            $("body").addClass("device-type-"+devType);
            $("body").addClass("device-version-"+osSplit[pointer]);
        }
    },
    _getTimestamp: function(useMicrotime){
        ((!useMicrotime)?useMicrotime=false:null);

        if (useMicrotime){
            var unix = Math.round(+new Date());
        }else{
            var unix = Math.round(+new Date()/1000);
        }
        
        return unix;
    },
    _registerForEvent: function(eventName,eventCallback){
        var self = this;
        if (_.isUndefined(self._registers[eventName])){
            self._registers[eventName] = [];
        }
        self._registers[eventName].push(eventCallback);
    },
    _callRegisterFunc: function(eventName,clearFunc){
        ((!clearFunc)?clearFunc=false:null);
        var self = this;
        if (!_.isUndefined(self._registers[eventName])){
            for(var i = 0;i<self._registers[eventName].length;i++){
                self._registers[eventName][i]();
            }
            if (clearFunc){
                self._registers[eventName] = [];
            }
        }
    },
    _callStorageReady: function(){
        var self = app;

        // check for deviceReady status
        if (!self._data.deviceReadyCalled){
            console.warn("app._callStorageReady: reject call - device is not ready!");
            return false;
        }

        // check for already called status
        if (self._data.storageReadyCalled){
            console.warn("app._callStorageReady: reject call - already called!");
            return false;
        }

        if (_.isUndefined(app.helper) || _.isUndefined(app.helper.storage) || _.isEmpty(app.helper.storage)){
            // check for app.init status - this will be an exception
            if (app._data.appInitCalled){
                eHandler.exception("Impossible to call 'storage._initReady'!",'app');
                return false;
            }
            console.warn("app._callStorageReady: reject call - app.helper.storage is unavailable!");
            return false;
        }

        // prevent double call        
        self._data.storageReadyCalled = true;

        // call READY for storageHelper
        app.helper.storage._initReady();

        return true;
    },
    _callSecureTimer: function(){
        window.setTimeout(function(){
            if (!app._data.storageReadyCalled){
                eHandler.exception("DeviceReady timeout!",'app');
            }
        },10000);
    },

    // empty containers for startup
	api: {},
	settings: {},
    system: {},
	nav: {},
	helper: {
        storage: {},
        nav: {},
    }
};