/******************
 * GLOBALS - DATA *
 ******************/

// the page model
var model = {};

// global vars
var GLOBALS = {
    appName: "dawanda",
    host: 'https://mobile.dawanda.com',
    includePath: '',
    debug: {
        status: false,
        lvl: 'error',
        send: false,
        receive: false
    },
    redirect: {},
    startup: {
        pageName: '#startup',
        pageParams: {noHistory: true}
    },
    'i18n': {
        status: true
    }
};


/********************
 * COMMON FUNCTIONS *
 ********************/

// HANDLES DEEPLINK TO APP (PLUGIN NATIVE FUNCTION)
function handleOpenURL(rawUrl) {    
    // TIMEOUT IS FIX FOR iOS, don't remove
    setTimeout(function() {
        // check for app url
        var decodedRawUrl = decodeURI(rawUrl);
        var urlPrefix = decodedRawUrl.substr(0, decodedRawUrl.indexOf('://')).toLowerCase();
        console.log("handleOpenURL: "+decodedRawUrl);
        if(urlPrefix == GLOBALS.appName){
            var urlParams = decodedRawUrl.substr(decodedRawUrl.indexOf('://')+3);
            var urlJSON = parse('{"' + urlParams.replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g,'":"') + '"}');
            GLOBALS.redirect = urlJSON;

            // check for existing app.nav
            if (_.isReady(app.nav)){
                app.nav.loadRedirect();
            }
        }
    }, 0);
}

// convert a jsonObj to URI
function uriConv(aObj,excludeData){
    ((!excludeData)?excludeData=[]:null);
    var _gParams = [];

    var _uConv = function(cObj,cPrefix){
        ((!cPrefix)?cPrefix="":null);

        if (_.isArray(cObj)){
            _(cObj).forEach(function(cData){
                if (_.isObject(cData)){
                    _uConv(cData,cPrefix);
                }else{
                    var cParam = encodeURIComponent(cPrefix) + '[]=' + encodeURIComponent(cData);
                    _gParams.push(cParam);
                }
            });
        }else{
            _.forEach(cObj,function(cData,cKey){
                if (_.contains(excludeData,cKey)){
                    return;
                }
                var tmpKey = cKey;
                if (cPrefix != ""){
                    tmpKey = cPrefix+"["+cKey+"]";
                }

                if (_.isObject(cData)){
                    _uConv(cData,cKey);
                }else{
                    var cParam = encodeURIComponent(tmpKey) + '=' + encodeURIComponent(cData);
                    _gParams.push(cParam);
                }
            });
        }
    };
    _uConv(aObj);
    return _gParams.join('&');
}

// parse a JSON string to obj
function parse(string) {
    var result = '';

    try {
        result = JSON.parse(string)
    }catch(e) {
        console.error('failed to parse json: ',string);
    }
    return result;
}

// stringify a JSON-obj to string
function stringify(object) {
    var result = '';

    try {
        result = JSON.stringify(object);
    }catch(e) {
        console.error('failed to stringify object:',object);
    }
    return result;
}

// load a HTTP-request file
function loadXmlHttpFile(src,successCallback,errorCallback,timeout){
    ((!successCallback)?successCallback=function(d){}:null);
    ((!errorCallback)?errorCallback=function(e){}:null);
    ((!timeout)?timeout=4000:null);

    var xmlHttpTimeout = null;

    try{
        var xmlHttp = new XMLHttpRequest();
        xmlHttp.open("GET", src, false);

        xmlHttp.onreadystatechange=function(){
            if (xmlHttp.readyState == 4 && (xmlHttp.status == 200 || xmlHttp.status == 0)) {
                clearTimeout(xmlHttpTimeout); 
                successCallback(xmlHttp.responseText);
            }else if (xmlHttp.readyState == 4 && xmlHttp.status == 404){
                clearTimeout(xmlHttpTimeout); 
                errorCallback();
            }
        }

        xmlHttpTimeout = setTimeout(function(){
            xmlHttp.abort();
            errorCallback();
        },timeout);
        xmlHttp.send();
    }catch(e){
        clearTimeout(xmlHttpTimeout);
        errorCallback(e);
    }
    return null;
}

// loadJSONXmlHttp
function loadJSON(src,successCallback,errorCallback){
    ((!successCallback)?successCallback=function(d){}:null);
    ((!errorCallback)?errorCallback=function(e){}:null);
    loadXmlHttpFile(
        src,
        function(d){
            successCallback(parse(d));
        },
        errorCallback
    );
}

// returns htmlCode
function loadHtml(src,printHtml){
    ((!printHtml)?printHtml=false:null);
    console.info("loadHtml: '"+src+"'");
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("GET", src, false);
    xmlHttp.send();
    if (printHtml){
        document.writeln(xmlHttp.responseText);
        return true;
    }
    return xmlHttp.responseText;
}

// render HTMLCode at this position
function renderTemplate(src){
    var $scriptNode = $('script').last();
    var scriptNode = $scriptNode.get(0);
    scriptNode.type = 'text/html';
    $scriptNode.load(src);
}

// loads a global script and appends it to the <head>
function loadScript(src) {
    loadXmlHttpFile(
        src,
        function(data){
            console.info("loadScript: '"+src+"'");
            var se = document.createElement('script');
            se.type = "text/javascript";
            se.text = data;
            document.getElementsByTagName('head')[0].appendChild(se);
        },function(){
            console.error("loadScript.error: '"+src+"'");
        }
    );
    return null;
}

// loads a global css and appends it to the <head>
function loadCSS(src) {
    console.info("loadCSS: '"+src+"'");
    var link = document.createElement('link');
    link.href = src;
    link.type = "text/css";
    link.rel = "stylesheet";
    document.getElementsByTagName("head")[0].appendChild(link);
}


function createCSSDataNode(data){
    console.info("createCSSDataNode");
    var link = document.createElement('style');
    link.innerHTML = data;
    link.type = "text/css";
    document.getElementsByTagName("head")[0].appendChild(link);
}

function createJSDataNode(data){
    console.info("createJSDataNode");
    var se = document.createElement('script');
    se.type = "text/javascript";
    se.text = data;
    document.getElementsByTagName('head')[0].appendChild(se);
}

// source: http://phpjs.org/functions/nl2br/
function nl2br(str, is_xhtml) {
    var breakTag = (is_xhtml || typeof is_xhtml === 'undefined') ? '<br ' + '/>' : '<br>';
    return (str + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + breakTag + '$2');
}

function formatString(inString){
    var rText = nl2br(inString);
    rText = rText.replace(/\*([^\*]+?)\*/g,'<b>$1</b>'); // bold => *text*
    rText = rText.replace(/_([^_]+?)_/g,'<u>$1</u>'); // underline => _text_
    rText = rText.replace(/\+([^\+]+?)\+/g,'<i>$1</i>'); // italic => +text+
    return rText;
}