/********************************************************************
 * navHelper                                                        *
 ********************************************************************
 * extend app.helper to handle iFrame-messages & window listeners  *
 ********************************************************************/

 (function(app){
    app.helper.nav = {
        // need this for eListener
        _className: 'navHelper',

        _tpl: {
            commReceive: "((typeof window.commHelper == 'undefined')?null:window.commHelper.receive())",
            commInject: "window.commHelper = {_cache: [],send:function(action,newdata){this._cache.push({'action': action, 'data': newdata});},receive:function(){return this._cache.splice(0);}}; ((typeof window.commHelperReady != 'undefined')?window.commHelperReady():null);"
        },
        _h: {
            'windows':      [],
            'iframes':      [],
        },
        'i18n': {
            init: function(localeString,successCallback){
                ((!successCallback)?successCallback=function(){}:null);
                var self = app.helper.nav;

                var locNamespaces = app.settings.get("i18n.nsList");

                var _initI18n = function(resourceStore){
                    $.i18n.init({
                        lng: localeString,
                        customLoad: function(lng, ns, options, loadComplete){
                            self.i18n.loadKeysFor(lng,ns,function(d){
                                if (!_.isEmpty(d)){
                                    console.log("i18n.success: '"+lng+"-"+ns+"'",d);
                                    loadComplete(null,d);
                                }else{
                                    console.error("i18n.error: '"+lng+"-"+ns+"'");
                                    loadComplete("i18n.error: not found '"+lng+"-"+ns+"'");
                                }
                            });
                        },
                        lowerCaseLng: true,
                        //resGetPath: GLOBALS.includePath+'i18n/__lng__/__ns__.json',       // disabled for 'async'-key loading
                        ns: {
                            namespaces: locNamespaces,
                            defaultNs: locNamespaces[0]
                        },
                        fallbackLng: 'en',
                        useLocalStorage: false,
                        debug: false,
                        getAsync: true
                    }, successCallback);
                };

                _initI18n();
            },
            loadKeysFor: function(localeString,locNamespace,successCallback){
                var mapResKey = "/i18n/"+localeString+"/"+locNamespace+".json";

                // getFileContent
                app.helper.storage.file.get(
                    mapResKey,
                    ':manifest:',   // get from manifest
                    function(d){
                        successCallback(parse(d));
                    },
                    function(e){
                        successCallback({});
                    }
                );
                return null;
            },
            changeLng: function(newLng,successCallback){
                $.i18n.setLng(newLng, successCallback);
            },
            updateDomNode: function(domNode){
                domNode.i18n();
                domNode.find('*[data-i18n-attr]').each(function (i) {
                    var nodeSelf = $(this);
                    var locAttrText = nodeSelf.attr("data-i18n-attr");
                    var attrs = locAttrText.split(",");
                    _(attrs).forEach(function(a){
                        var attrToken = a.split(":");
                        var attrKey = attrToken[0].trim();
                        var attrValue = attrToken[1].trim();
                        nodeSelf.attr(attrKey,i18n.t(attrValue));
                    });
                });
            }
        },
        'iframe': {
            history: {
                'isBackable': function(cIframeObj){
                    var helper = app.helper.nav.iframe;
                    var cIframe = helper._fW(cIframeObj);

                    if (!cIframe){
                        return false;
                    }

                    var cHistoryLength = cIframe.window.contentWindow.history.length;
                    if (cHistoryLength != cIframe.history.l){
                        cIframe.history.l = cHistoryLength;
                        cIframe.history.c = cIframe.history.l;
                    }

                    if (app.settings.get("PLATFORM") != "ios" && cIframe.history.c > 1){
                        return true;
                    }else if(app.settings.get("PLATFORM") == "ios" && cIframe.history.c > 2){
                    	return true;
                    }else{
                        return false;
                    }
                },
                'goBack': function(cIframeObj){
                    var helper = app.helper.nav.iframe;
                    var cIframe = helper._fW(cIframeObj);
                    console.warn("REDUCE HISTORY: "+cIframe.history.c);
                    cIframe.history.c--;
                    return cIframe.window.contentWindow.history.back();
                }
            },
            // SCOPE: {type: 'iframe|window', 'id': <ID>, action: <ACTION>, data: <DATA>}
            register: function(iFrameObj,iFrameDomain,iCallbacks){
                ((!iCallbacks)?iCallbacks={}:null);
                var self = app.helper.nav;
                var helper = this;

                // get new iFrameId
                var cIframeId = self._h.iframes.length;

                // set new iFrameObj
                var cIframeObj = {id: cIframeId, window:iFrameObj, domain: iFrameDomain, 'interval':null,'callbacks': iCallbacks, 'history': {c: 0, l: 0}};
                // push new window to history
                self._h.iframes.push(cIframeObj);

                // register 'loadstop' callback - loading of a new page is done (docReady)
                cIframeObj.window.onload = function(){
                    // register
                    helper.call(cIframeObj,"register");
                };            

                helper._execCallback(cIframeObj,'setup');
                return cIframeId;
            },
            start: function(cIframeObj){
                var self = app.helper.nav;
                var helper = this;
                var cIframe = helper._fW(cIframeObj);

                clearInterval(cIframe.interval);
                cIframe.interval = window.setInterval(function(){
                    helper.call(cIframeObj,'receive');
                },500);
            },
            stop: function(cIframeObj){
                var helper = this;
                var cIframe = helper._fW(cIframeObj);
                clearInterval(cIframe.interval);
            },
            call: function(cIframeObj,callAction,callParams){
                ((!callParams)?callParams={}:null);
                var helper = this;
                var cIframe = helper._fW(cIframeObj);

                var _scope = {id: cIframe.id, type: 'iframe', action: callAction, data: callParams};
                cIframe.window.contentWindow.postMessage(_scope,cIframe.domain);
            },
            cb: function(cIframeObj,cbAction,cbParams){
                var helper = this;
                var cIframe = helper._fW(cIframeObj);
                helper._execCallback(cIframe,cbAction,cbParams);
            },
            _fW: function(wParam){
                var self = app.helper.nav;
                if (_.isObject(wParam)){
                    return wParam;
                }else if(_.isNumber(wParam) && !_.isUndefined(self._h.iframes[wParam])){
                    return self._h.iframes[wParam];
                }else{
                    return null;
                }
            },
            _execCallback: function(cIframeObj,cbAction,cbParams){
                ((!cbParams)?cbParams={}:null);
                if (!_.isUndefined(cIframeObj.callbacks[cbAction])){
                    cIframeObj.callbacks[cbAction](cbParams);
                }
            }
        },
        'window': {
            open: function(wUrl,wParams,wCallbacks){
                ((!wParams)?wParams={}:null);
                ((!wCallbacks)?wCallbacks={}:null);
                var self = app.helper.nav;
                var helper = this;

                // set target
                if (_.isUndefined(wParams["target"])){
                    wParams["target"] = "_blank";
                }
                // set iOS defaults
                if (_.isUndefined(wParams["location"])){
                    wParams["location"] = "no";
                }
                if (_.isUndefined(wParams["toolbar"])){
                    wParams["toolbar"] = "no";
                }

                // implode elements to paramString
                var paramArray = [];
                _.forEach(wParams,function(pValue,pKey){
                    if (pKey == "target"){
                        return;
                    }
                    paramArray.push(pKey+"="+pValue);
                });

                // get new windowId
                var cWindowId = self._h.windows.length;

                // set new windowObj
                var cWindowObj = {id: cWindowId, window:null, 'interval':null,'status': "starting",'callbacks': wCallbacks};

                // open new window
                cWindowObj.window = window.open(wUrl, wParams.target, paramArray.join(','));

                // register 'loadstop' callback - loading of a new page is done (docReady)
                cWindowObj.window.addEventListener('loadstop',function(){
                    helper._execCallback(cWindowObj,'loadstop');
                    // call run function
                    helper.start(cWindowObj);
                });

                // register 'loadstart' callback - loading of a new page is started (blur old page)
                cWindowObj.window.addEventListener('loadstart',function(){                  
                    // call run function
                    helper.stop(cWindowObj);
                    helper._execCallback(cWindowObj,'loadstart');
                });

                // register 'loaderror' callback - loading of a new page has errors
                cWindowObj.window.addEventListener('loaderror',function(){
                    helper._execCallback(cWindowObj,'loaderror');
                });

                // register 'exit' callback - window will close
                cWindowObj.window.addEventListener('exit',function(){
                    helper._execCallback(cWindowObj,'exit');
                });

                // push new window to history
                self._h.windows.push(cWindowObj);
                helper._execCallback(cWindowObj,'setup');
                return cWindowId;
            },
            close: function(cWindowObj){
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                cWindow.status = "exiting";
                helper.stop(cWindowObj);
                cWindow.window.close();
            },
            start: function(cWindowObj){
                var self = app.helper.nav;
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                if (cWindow.status == "exiting"){
                    return false;
                }
                cWindow.interval = window.setInterval(function(){
                    helper.call(cWindowObj,self._tpl.commReceive,function(d){
                        if((_.isNull(d[0]) || _.isUndefined(d[0])) ){
                            helper.inject(cWindowObj);
                        }
                        else if(cWindow.status == "reading"){
                            helper.read(cWindowObj,d[0]);
                        }
                    });
                },100);
            },
            stop: function(cWindowObj){
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                clearInterval(cWindow.interval);
                cWindow.status = 'starting';
            },
            call: function(cWindowObj,callCode,callReceiveCallback){
                ((!callReceiveCallback)?callReceiveCallback=function(d){}:null);
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                cWindow.window.executeScript({code: callCode},callReceiveCallback);
            },
            inject: function(cWindowObj){
                var self = app.helper.nav;
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                if (cWindow.status == "exiting"){
                    return false;
                }

                helper.call(cWindowObj,self._tpl.commInject,function(d){
                    helper._execCallback(cWindowObj,'init');
                    cWindow.status = "reading";
                });
            },
            read: function(cWindowObj,readData){
                var helper = this;

                // check every action
                _(readData).forEach(function(dObj){
                    switch (dObj.action){
                        case 'close':
                            helper.close(cWindowObj);
                        break;
                        default:
                            helper._execCallback(cWindowObj,dObj.action,dObj.data);
                        break;
                    }
                });
            },
            cb: function(cWindowObj,cbAction,cbParams){
                var helper = this;
                var cWindow = helper._fW(cWindowObj);
                helper._execCallback(cWindow,cbAction,cbParams);
            },
            _fW: function(wParam){
                var self = app.helper.nav;
                if (_.isObject(wParam)){
                    return wParam;
                }else if(_.isNumber(wParam) && !_.isUndefined(self._h.windows[wParam])){
                    return self._h.windows[wParam];
                }else{
                    return null;
                }
            },
            _execCallback: function(cWindowObj,cbAction,cbParams){
                ((!cbParams)?cbParams={}:null);

                if (!_.isUndefined(cWindowObj.callbacks[cbAction])){
                    cWindowObj.callbacks[cbAction](cbParams);
                }
            }
        },
        _init: function(){
            var self = this;
            // register READY
            eListener.register(["nav","system"],app.helper.nav._initReady);
        },
        _initReady: function(){
            // reference to global - unable to load through eListener ('this' is eListener)
            var self = app.helper.nav;


            // check for i18n
            if (GLOBALS.i18n.status){
                self.i18n.init(app.settings.get("LANGUAGE"),function(){
                    _.setReady(self);
                });
            }else{
                _.setReady(self);
            }
        }
    };
	
    // init the logHelper
	app.helper.nav._init();
})(window.app);