(function(app){
	app.rus = {
		// need this for eListener
		_className: 'rus',

		// setup the system - first start after app download & install
		systemSetup: function(successCallback,errorCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			var self = this;

			var _setupStorage = function(sC,eC){
				app.helper.storage._setup(
					function(d){	// successCallback
						// try load data
						app.settings.load(
							true,
							function(){
								app.settings.set("SETUP",true);
								app.settings.save(sC,eC);
							},
							eC
						);
					},
					eC
				);
			};

			_setupStorage(successCallback,errorCallback);
			return null;
		},

		systemManifestCheck: function(successCallback,errorCallback){
			var midConst = app.settings.get("api.mid");
			var midCurrent = app.settings.get("api.mid.setup");
			if (midConst > midCurrent){
				// delete current manifest
				app.settings.set("MANIFEST","",true,function(){
					// load manifest from storage
					// MANIFEST
					loadJSON(app.settings.get("FMAP.manifest"),function(d){
						app.settings.set("MANIFEST",d, true);
						app.settings.set("REV","r0",true);
						app.settings.set("api.mid.setup",midConst,true,successCallback,errorCallback);
					});
				},errorCallback);
			}else{
				successCallback();
			}
		},

		systemUpdate: function(successCallback,errorCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			var self = this;

			var _newManifest = {};
			var _jobList = [];
			var _stats = {
				total: 0,
				current: 0
			};

			// get manifestData from API
			var _getManifest = function(sC,eC){
				app.api.doRequestAsync(":manifest",{},'get',sC,eC);
			};

			var _checkNewManifest = function(mData){
				// update is required
				if (mData.uStatus){
					// set global (this container)
					_newManifest = mData;

					// prepare update VIEW
					//_updateView.prepare();

					// create new joblist
					var jobList = _createJobList();

					// handle joblist
					_handleJoblist(jobList);

					// AUTO '_updateManifest'
				}
				// no update required, this is the newest version
				else{
					// FINAL - CALL SUCCESS
					successCallback();
				}
			};

			var _updateManifest = function(){
				// update settings with new data from manifest
				_updateSettings();

				// set new manifest
				app.settings.set("MANIFEST",_newManifest);

				// set new revision
				app.settings.set("REV",_newManifest.cRev);

				// FINAL - SAVE NEW MANIFEST
				app.settings.save(successCallback,errorCallback);
			}

			var _updateSettings = function(){
				if ((!_.isUndefined(_newManifest.dataSettings)) && (!_.isEmpty(_newManifest.dataSettings))){
					_.forEach(_newManifest.dataSettings,function(dValue,dKey){
						app.settings.set(dKey,dValue);
					});
				}
				return true;
			};

			var _createJobList = function(){
				var jobList = [];
				var cManifest = app.settings.get("MANIFEST");

				var _preparedFileData = {};
				// SINGLE-JOB: {action: 'remove|set',type: 'sql,fs', key: '<KEY>', size: 0}

				// preperate oldManifest to get access to all files-by-afid
				_.forEach(cManifest.dataFiles,function(fData){
					_preparedFileData[fData.dataAfid] = fData;
				});

				// NEW MANIFEST - check data
				_.forEach(_newManifest.dataFiles,function(fData){
					// prepare oldFileData dummy
					var oldFdata = {dataKey: null, dataTimestamp: 0, dataStorageType: 'undefined'};
					var doUpdateJob = false;
					var doDeleteJob = false;
					var fileHasChanged = false;

					// prevent 'null' keys
					if (_.isNull(fData.dataKey) || _.isUndefined(fData.dataKey)){
						return;
					}

					// fill oldFileData with dataContent
					if (!_.isUndefined(_preparedFileData[fData.dataAfid])){
						oldFdata = _preparedFileData[fData.dataAfid];
					}

					// check for changed timestamps
					if (oldFdata.dataTimestamp < fData.dataTimestamp){
						fileHasChanged = true;
					}

					// check for changed storageLocations
					if (oldFdata.dataStorageType != 'undefined' && oldFdata.dataStorageType != fData.dataStorageType){
						fileHasChanged = true;
					}


					// file has changed
					if (fileHasChanged){
						// check for old storage type to delete
						if (oldFdata.dataStorageType != fData.dataStorageType && (oldFdata.dataStorageType == "fs" || oldFdata.dataStorageType == "sql")){
							doDeleteJob = true;
						}

						// check for new storage type to update
						if (fData.dataStorageType == "fs" || fData.dataStorageType == "sql"){
							doUpdateJob = true;
						}
					}

					// ----------- HANDLE GOES HERE -----------

					// check for update job
					if (doUpdateJob){
						jobList.push({action: 'set', type: fData.dataStorageType, key: fData.dataKey, size: fData.dataSize});
						// add new filesize to stats
						_stats.total += parseInt(fData.dataSize);
					}

					// check for remove old job
					if (doDeleteJob && (!_.isNull(oldFdata.dataKey))){
						jobList.push({action: 'remove', type: oldFdata.dataStorageType, key: oldFdata.dataKey});
					}

					// remove entry from _preparedFileData
					if (!_.isNull(oldFdata.dataAfid)){
						delete _preparedFileData[oldFdata.dataAfid];
					}
				});

				// DELETE rest of old files
				_.forEach(_preparedFileData,function(fData){
					if (fData.dataStorageType != "online" && fData.dataStorageType != "app"){
						jobList.push({action: 'remove', type: fData.dataStorageType, key: fData.dataKey});
					}
				});
				return jobList;
			};

			var _handleJoblist = function(cJobList){
				// set global (this container)
				_jobList = cJobList.reverse();

				// prepare view for update
				_updateView.prepare();

				// wait a sec for optimized user experience
				window.setTimeout(function(){
					// start with first job
					_handleSingleJob(_jobList.pop());
				},1000);
			};

			var _handleSingleJob = function(jobData){
				// break condition
				if (_.isNull(jobData) || _.isEmpty(jobData)){
					_updateManifest();
					return;
				}

				switch (jobData.action){
					case 'remove':
						console.log("rus.job.remove: "+jobData.key,jobData.type);
						app.helper.storage.file.remove(
							jobData.key,
							jobData.type,
							function(s){
								// start with next job
								_handleSingleJob(_jobList.pop());
							},
							// ignore file remove errors
							function(e){
								console.error("file remove error: "+jobData.key+" (type: "+jobData.type+")",e);

								// start with next job
								_handleSingleJob(_jobList.pop());
							}
						);
					break;
					case 'set':
						// calc stats
						var jobSize = parseInt(jobData.size);
						var partPerc = Math.round(((jobSize/_stats.total)*100*2)/2);
						console.log("rus.job.set: "+jobData.key,jobData.type);
						app.api.doRequestBinaryAsync(
							":file",
							{afid: jobData.key},
							function(fileData){
								if (_.isObject(fileData) && fileData===null){
									errorCallback("file is empty: "+jobData.key);
									return;
								}
								// update stats
								_updateView.addPerc(partPerc,true);
								_stats.current += jobSize;

								app.helper.storage.file.set(
									jobData.key,
									jobData.type,
									fileData,
									function(s){
										// start with next job
										_handleSingleJob(_jobList.pop());
									},
									errorCallback
								);
							},
							errorCallback,
							function(progPerc){
								var pCurrent = Math.round((partPerc*progPerc*2)/2);
								_updateView.percTmp(pCurrent,true)
							}
						);
					break;
					default:
						// start with next job
						_handleSingleJob(_jobList.pop());
					break;
				}
			};

			// this is the update VIEW, show the user, that the system ist updating...
			var _updateView = {
				_cPerc: 0,
				_cPercTmp: 0,
				'prepare': function(){
					// toDo: change to nav
					app.nav.showContainer('update');
				},
				'msg': function(msg){
					$("#update_message p:first").html(msg);
				},
				'progress': function(perc){
					$("#update_progress_container .bar .barProgress").stop().animate({
						width: perc+"%"
					},{
						step : function(now, fx){
			                $("#update_progress_container .progressText").html(Math.ceil(now)+" %");
			            },duration: 400
					});
				},
				'addPerc': function(perc,progressView){
					((!progressView)?progressView=false:null);

					this._cPerc += perc;

					// prevent > 100
					((this._cPerc>100)?this._cPerc=100:null);

					if (progressView){
						this.progress(this._cPerc);
					}
				},
				'percTmp': function(perc,progressView){
					((!progressView)?progressView=false:null);

					this._cPercTmp = perc;

					if (progressView){
						var tmpTotal = this._cPerc+this._cPercTmp;

						// prevent > 100
						((tmpTotal>100)?tmpTotal=100:null);
						
						this.progress(tmpTotal);
					}
				}
			};

			// start over here...
			_getManifest(
				_checkNewManifest,
				function(jqXHR){
					if(jqXHR.statusText=="timeout"){
						app.nav.showContainer("offline",true);
					}
					else{
						errorCallback();
					}
				}
			);
		},
		_init: function(){
			var self = this;
			// register READY
			eListener.register(["storageHelper","api","settings"],app.rus._initReady);
		},
		_initReady: function(){
			// reference to global - unable to load through eListener (this is eListener)
			var self = app.rus;

			// register breakPoint, for errors
			var _registerBreakPoint = function(e){
				((!e)?e="":null);
				app._registerForEvent('breakPoint',function(){
					app.rus._initReady();
				});

				// check for offline dev-data
				if (e != ""){
					window.setTimeout(function(){
						if (!app.settings.get("OFFLINE")){
							eHandler.exception(e,self);
						}
					},5000);
				}
			};

			var _sysReady = function(){
				app.settings.set("MCHECK",true);
				_.setReady(self);
			};

			var _systemHold = function(e){
				// break point for connection lost problems
				// show alternative: error
				_registerBreakPoint(e);
			};

			var _sysManifestCheck = function(){
				self.systemManifestCheck(_sysUpdate,_systemHold)
			}

			var _sysUpdate = function(){
				self.systemUpdate(_sysReady,_systemHold);
			};

			var _sysSetup = function(){
				self.systemSetup(_sysUpdate,_systemHold);
			};

			/*****************
			 * RUN GO'S HERE *
			 *****************/

			// check for ready state, don't execute twice
			if (_.isReady(self)){
				return;
			}

			// setup the system
			if (!app.settings.get("SETUP")){
				_sysSetup();
			}else if (!app.settings.get("MCHECK")){
				// do a ManifestCheck (request API for updates)
				_sysManifestCheck();
			}else{
				_sysReady();
			}
		}
	};
	app.rus._init();
})(window.app);