/********************************************************************
 * app.settings														*
 ********************************************************************
 * read/write settings and load/store data from/to localStorage		*
 ********************************************************************/

(function(app){
	app.settings = {
		// need this for eListener
		_className: 'settings',

		// the temporary data (loaded & stored in the SQL-TBL [storageHelper])
		_data: {},
		
		// the 'static' data (not stored - do not change manually)
		_const: {
			loaded: false
		},
		
		// set the default data
		setDefaults: function(){
			var self = this;
			// CONST VALUES - NOT STORED
			self.setConst("ENV","DEV");										// environment ('DEV','PRO')
			self.setConst("PLATFORM",cordova.platformId.toLowerCase());		// platform ('android','ios')
			self.setConst("PLATFORM_VERSION",'unknown');					// platform version - updated in 'updateDefaults'
			self.setConst("LOCALE",navigator.language);						// locale 'de-DE, en-EN'

			// STATIC - FILE MAPPING
			self.setConst("FMAP.manifest","shared/data/manifest.json");		// this is the static-path for the manifest-file
			self.setConst("FMAP.analytics","/shared/data/ga_mapping.json");	// this is the static-path for the ga file
			
			// STATIC - AUTO CHANGED BY SYSTEM FNC
			self.setConst("MCHECK",false);									// manifest checked? (RUS)
			self.setConst("STARTUP",false);									// startup already fired?
			self.setConst("REF_PREFIX","#app:");							// reference-prefix (to locate dyn files)
			self.setConst("RESTART_TIMER",172800);							// restart the app after timeout of x (172800 <=> 2 days)

			// DEFAULT VALUES - STORED
			self.set("REV","r0");											// the current revision (manifest version)
			self.set("SETUP",false);										// setup done? (false for the very first startup)
			self.set("LANGUAGE",navigator.language.split("-")[0]);			// locale 'de, en' 
			self.set("CURRENCY","eur");										// currency 'eur', 'gbp' or 'pln'

			// i18n
			self.set("i18n.nsList",['app','countries']);					// NameSpace list for i18n

			// DEBUG - ONLY FOR DEV-MODE
			self.setConst("debug.host",GLOBALS.host);
			self.set("debug.id",uuid(8));
			self.set("debug.status",GLOBALS.debug.status);
			self.set("debug.lvl",GLOBALS.debug.lvl);
			self.set("debug.receiveInterval",2000);
			self.set("debug.send",GLOBALS.debug.send);
			self.set("debug.receive",GLOBALS.debug.receive);

			// ANALYTICS
			self.setConst("analytics.id","UA-76434-46");
			self.set("analytics.status",true);

			// MANIFEST
			loadJSON(self.get("FMAP.manifest"),function(d){					// loads the 'app-manifest'
				self.set("MANIFEST",d);
			});

			// API CONST VALUES - NOT STORED
			self.setConst("api.mstatus","public");							// the APP TYPE (public | private)
			self.setConst("api.mid","1.5");									// the VERSION of this FILE (NOTE: only <A>.<B> - not a.b.c !)
			self.setConst("api.host",GLOBALS.host);							// the API-hostname

			// API VALUES
			self.set("api.mid.setup","1.5.0");								// the current VERSION (NOTE: must be <A>.<B>.<C>)
			self.set("api.timeout",7000);
			self.set("api.sharedSecret","uIjKmN27Gc");
			self.set("api.sessionToken","");
			self.set("api.rememberToken","");
			self.set("api.userLoggedIn",false);
			self.set("api.userName","");

			self.set("api.targets",{
				':manifest': '/manifest',
				':file': '/manifest/file',
				':api': '/api/r',
				':apiPost': '/api/rp',
				':apiDelete': '/api/rd',
				':apiLoginFB': '/api/login_fb',
				':apiLoginDW': '/api/login_dw',
				':apiRegisterDW': '/api/register_dw',
				':apiResendEmail': '/api/resend_validation_email',
				':apiPasswordReset': '/api/password_reset',
				':apiGetCurrency': '/api/getcurrency',
				':pnRegister': '/pn/register',
				':pnDelete': '/pn/delete',
				':support': '/dev/support'
			});

			// pn settings
			self.set("pn.channels.reminders",false);
			self.set("pn.channels.inspirations",false);
			self.set("pn.channels.highlights",false);
			self.set("pn.hasBeenAsked",false);
			self.set("pn.deviceToken",false);

			// SYSTEM CONST VALUES - NOT STORED
			self.setConst("dataStorage","dataDirectory");
			self.setConst("sqlDatabaseName","sqldatabase");
			self.setConst("sqlDatabaseSize",50*1024*1024);
		},

		// update defaults data, dependent on deviceReady function
		updateDefaults: function(){
			var self = this;

			self.setConst("PLATFORM_VERSION",((typeof device != 'undefined')?device.version:'unknown'));	// platform version ('8.0')
		},
		
		// returns the 'value' (if 'key' is setted)
		get: function(key){
			var self = this;

			// check const
			if (!_.isUndefined(self._const[key])){
				return self._const[key];
			}

			// check data
			if (!_.isUndefined(self._data[key])){
				return self._data[key];
			}
			
            return null;
		},

		// returns all values
		getAll: function(eType){
			((!eType)?eType="data":null);
			var self = this;

			if (eType == "data"){
				return self._data;
			}

			if (eType == "const"){
				return self._const;
			}

            return null;
		},
		
		// sets the key and it's value
		set: function(key, value,saveSingleValue,saveSuccessCallback,saveErrorCallback){
			((!saveSuccessCallback)?saveSuccessCallback=function(d){}:null);
			((!saveErrorCallback)?saveErrorCallback=function(d){}:null);
			((!saveSingleValue)?saveSingleValue=false:null);
			var self = this;

			// don't save to SQL, but overwrite const data
			if (self.isConst(key)){
				self._const[key] = value;
				return false;
			}
			// set new data
            self._data[key] = value;

            // save single value to storage
            if (saveSingleValue){
            	app.helper.storage.setting.set(key,value,saveSuccessCallback,saveErrorCallback);
            }
            return true;
		},
		// sets a constant key/value but doesn't store this
		setConst: function(key,value){
			var self = this;	
            self._const[key] = value;
            return true;
		},
		
		// load settings from database
		load: function(force,successCallback,errorCallback){
			((!force)?force=false:null);
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			var self = this;
            
            if (force || !self.get("loaded")) {
            	// verify app.helper.storage is ready
            	if (!_.isReady(app.helper.storage)){
            		errorCallback('storageHelper.notready');
            		return false;
            	}

            	// get settings from storageHelper
            	app.helper.storage.setting.getAll(function(storedData){
            		if(!_.isEmpty(storedData)){
						self._data = storedData;
					}
					self.setConst("loaded",true);

            		successCallback();
            	},function(e){
            		errorCallback(e);
            	});
            }
            return null;
		},
		
		// saves data to localStorage
		save: function(successCallback,errorCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			var self = this;

			// verify app.helper.storage is ready
        	if (!_.isReady(app.helper.storage)){
        		errorCallback('storageHelper.notready');
        		return false;
        	}

        	// set settings to storageHelper
        	app.helper.storage.setting.setAll(self._data,successCallback,errorCallback);
            return null;
		},
		
		// check's, if a key exists
		isSet: function(key,checkConst){
			((!checkConst)?checkConst=false:null);
			var self = this;

            if (_.isUndefined(self._data[key])){
            	// check const
            	if (!_.isUndefined(self._const[key])){
            		return true;
            	}
            	return false;
            }
            return true;
		},

		// check's, if a const exists
		isConst: function(key){
			var self = this;

            if (!_.isUndefined(self._const[key])){
        		return true;
        	}
        	return false;
		},
		
		// resets default data (and load data from localStorage)
		restoreDefaults: function(){
			var self = this;
			self.setDefaults();
			self.load(true);
		},

		// resets all stored data and load defaultData
		reset: function(successCallback,errorCallback){
			((!successCallback)?successCallback=function(d){}:null);
			((!errorCallback)?errorCallback=function(e){}:null);
			var self = this;

			var _resetData = function(){
				self.restoreDefaults();
				successCallback();
			};
			app.helper.storage.setting.truncate(_resetData,errorCallback);
			return null;
		},
		
		/****************
		 * PRIVATE AREA *
		 ****************/

		 // init the settingsObj
		_init: function(){
			var self = this;
			// set defaults data
			self.setDefaults();

			// register READY function
			eListener.register("storageHelper",self._initReady);
		},
		
		// init the settingsObj
		_initReady: function(){
			// reference to global - unable to load through eListener (this is eListener)
			var self = app.settings;

			// update defaults data, dependent on deviceReady function
			self.updateDefaults();

			// load settings from DB (don't force - try it first time)
			self.load(false,function(){
				_.setReady(self);
			},function(e){
				// error callback
				// WARNING: Also set ready at the eCallback, the RUS need this
				// otherwise we will never have a startup, setup, update ...
				_.setReady(self);

				// @deprecated
				//eHandler.error(self,e);
			});
		}
	};
	app.settings._init();
})(window.app);