(function(app){
	app.helper.storage = {
		// need this for eListener
		_className: 'storageHelper',

		// GLOBAL VARS
		database: null,
		dataStorage: null,
		applicationStorage: {},

		// START SETTING-CONSTRUCT
		setting: {
			// 'returns' the value of 'key'
			get: function(key,successCallback,errorCallback){
				((!successCallback)?successCallback=function(d){}:null);
				((!errorCallback)?errorCallback=function(e){}:null);
				var self = app.helper.storage;

				// validate database
				if (_.isNull(self.database)){
					return errorCallback("storageHelper.get: database.notready");
				}

				// call database
				self.database.transaction(
					function(tx){
						tx.executeSql(
							'SELECT * FROM settings WHERE dataKey=? LIMIT 1;',
							[key],
							function(tx, result){
								// result success
								console.info("storageHelper-setting.get: ",result.rows);
								if(result.rows.length == 0){
									successCallback(null);
								}
								else{
									successCallback(parse(result.rows.item(0).dataValue));
								}
							},
							errorCallback
						);
					}
				);
			},
			// 'returns' a key:value obj
			getAll: function(successCallback,errorCallback){
				((!successCallback)?successCallback=function(d){}:null);
				((!errorCallback)?errorCallback=function(e){}:null);
				var self = app.helper.storage;

				// validate database
				if (_.isNull(self.database)){
					return errorCallback("storageHelper.getAll: database.notready");
				}

				// call database
				self.database.transaction(
					function(tx){
						tx.executeSql(
							'SELECT * FROM settings;',
							[],
							function(tx, result){
								var _dataObj = {};

								for (var i=0;i<result.rows.length;i++){
									var rData = result.rows.item(i);
									_dataObj[rData.dataKey] = parse(rData.dataValue);
								}
								console.info("storageHelper-setting.getAll: ",_dataObj);
								successCallback(_dataObj);
							},
							errorCallback
						);
					}
				);
			},
			// 'sets' the value of 'key'
			set: function(key, value, successCallback, errorCallback){
				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);
				var self = app.helper.storage;

				self.setting.get(
					key,
					function(result){
						if(result===null){
							self.database.transaction(
								function(tx){
									tx.executeSql(
										'INSERT INTO settings (dataKey, dataValue) VALUES (?,?);',
										[key,stringify(value)],
										function(tx, result){
											console.info("storageHelper-setting.set: "+key,value);
											successCallback(result);
										},
										errorCallback
									);
								}
							);
						}
						else{
							self.database.transaction(
								function(tx){
									tx.executeSql(
										'UPDATE settings SET dataValue=? WHERE dataKey=?;',
										[stringify(value),key],
										function(tx, result){
											console.info("storageHelper-setting.set: "+key,value);
											successCallback(result);
										},
										errorCallback
									);
								}
							);
						}
					},
					function(error){
						errorCallback
					}
				);
			},
			setAll: function(newData,successCallback,errorCallback){
				((!successCallback)?successCallback=function(d){}:null);
				((!errorCallback)?errorCallback=function(e){}:null);
				var self = app.helper.storage;

				// validate database
				if (_.isNull(self.database)){
					return errorCallback("database not ready");
				}

				// prepare data
				var dJobs = [];
				_.forEach(newData,function(dValue,dKey){
					dJobs.push({key: dKey, 'value': dValue});
				});


				var _singleStep = function(jobData){
					// break condition
					if (_.isNull(jobData) || _.isEmpty(jobData)){
						successCallback();
						return;
					}

					self.setting.set(
						jobData.key,
						jobData.value,
						function(d){
							// next job
							_singleStep(dJobs.pop());
						},
						errorCallback
					);
				};

				// first job
				_singleStep(dJobs.pop());
			},
			// 'sets' the value of 'key'
			remove: function(key, successCallback, errorCallback){
				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);
				var self = app.helper.storage;

				self.database.transaction(
					function(tx){
						tx.executeSql(
							'DELETE FROM settings WHERE dataKey=?',
							[key],
							function(tx, result){
								console.info("storageHelper-setting.remove: "+key);
								successCallback(result);
							},
							errorCallback
						);
					}
				);
			},
			// 'sets' the value of 'key'
			truncate: function(successCallback, errorCallback){
				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);
				var self = app.helper.storage;

				self.database.transaction(
					function(tx){
						tx.executeSql(
							'DELETE FROM settings;',
							[],
							function(tx, result){
								console.info("storageHelper-setting.truncate: ");
								successCallback(result);
							},
							errorCallback
						);
					}
				);
			}
		},

		file: {

			// gets a File from a defined or 'undefined' storageType
			// if sT is not defined, locations sql, fs, app and online are checked
			// if sT is ':manifest:', the manifest will be checked for the correct sT
			// returns null if file is not found
			get: function(key, storageType, successCallback, errorCallback){				
				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);
				var self = app.helper.storage;

				// check special storageType ':manifest'
				if (storageType == ":manifest:" || storageType == ":man:"){
					// reset storage type
					storageType = 'app';

					// get storage data from manifest
					var mData = self._getManifestDataForKey(key);
					if (!_.isEmpty(mData)){
						storageType = mData.dataStorageType;
					}
				}

				// if storageType is not defined, each storageType will be checked
				if(!storageType){
					// storygeType sql will be checked
					self.file.is(
						key,
						'sql',
						function(result){
							if(result){
								self.file.helper._getFromSql(
									key,
									successCallback,
									errorCallback
								);
							}
							else{
								// storygeType fs will be checked
								self.file.is(
									key,
									'fs',
									function(result){
										if(result){
											self.file.helper._getFromFs(
												key,
												successCallback,
												errorCallback
											);
										}
										else{
											self.file.is(
												key,
												'app',
												function(result){
													if(result){
														//get from app
														self.file.helper._getFromApp(
															key,
															successCallback,
															errorCallback
														);
													}
													else{
														self.file.helper._getFromOnline(
															key,
															successCallback,
															errorCallback
														);
													}
												},
												errorCallback
											);
										}
									}
								);
							}
						}
					);
				}
				// storygeType sql will be checked
				else if(storageType=='sql'){
					self.file.helper._getFromSql(
						key,
						successCallback,
						errorCallback
					);
				}
				// storygeType fs will be checked
				else if(storageType=='fs'){
					self.file.helper._getFromFs(
						key,
						successCallback,
						errorCallback
					);
				}
				// storygeType app will be checked
				else if(storageType=='app'){
					self.file.helper._getFromApp(
						key,
						successCallback,
						errorCallback
					);
				}
				// storygeType online will be checked
				else if(storageType=='online'){
					self.file.helper._getFromOnline(
						key,
						successCallback,
						errorCallback
					);
				}
				else{
					errorCallback('app.helper.storage.file.get: wrong storageType Parameter');
				}
			},

			set: function(key, storageType, fileContent, successCallback, errorCallback){

				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);

				var self = app.helper.storage;

				// if storageType is not defined, storageType sql will be selected
				if(!storageType || storageType=='sql'){
					// storygeType sql will be checked
					self.file.helper._setToSql(
						key,
						fileContent,
						successCallback,
						errorCallback);
				}
				// storygeType fs will be selected
				else if(storageType=='fs'){
					self.file.helper._setToFs(
						key,
						fileContent,
						successCallback,
						errorCallback
					);
				}
				// storygeType app will be selected
				else if(storageType=='app'){
					errorCallback('app.helper.storage.file.set: storageType app not supported');
				}
				// storygeType online will be selected
				else if(storageType=='online'){
					errorCallback('app.helper.storage.file.set: storageType online not supported');
				}
				else{
					errorCallback('app.helper.storage.file.set: wrong storageType Parameter');
				}
			},
			
			remove: function(key, storageType, successCallback, errorCallback){

				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);

				var self = app.helper.storage;

				// if storageType is not defined, storageType sql will be selected
				if(!storageType){
					// storygeType sql will be checked
					self.file.helper._isOnSql(key, function(result){
						if(result){
							self.file.helper._removeFromSql(
								key,
								function(){
									self.file.helper._isFileOnFs(
										key, function(result){
										if(result){
											self.file.helper._removeFileFromFs(key, successCallback, errorCallback);
										}
										else{
											successCallback(true);
										}
									}, errorCallback);
								},
								errorCallback
							);
						}
						else{
							self.file.helper._isFileOnFs(key, function(result){
								if(result){
									self.file.helper._removeFileFromFs(key, successCallback, errorCallback);
								}
								else{
									successCallback(false);
								}
							}, errorCallback);
						}
					}, errorCallback);
				}
				// storygeType fs will be selected
				else if(storageType=='sql'){
					self.file.helper._isOnSql(key, function(result){
						if(result){
							self.file.helper._removeFromSql(key, successCallback, errorCallback);
						}
						else{
							successCallback(false);
						}
					}, errorCallback);
				}
				// storygeType fs will be selected
				else if(storageType=='fs'){
					self.file.helper._isOnFs(key, function(result){
						if(result){
							self.file.helper._removeFileFromFs(key, successCallback, errorCallback);
						}
						else{
							successCallback(false);
						}
					}, errorCallback);
				}
				else if(storageType=='app'){
					errorCallback('app.helper.storage.file.remove: storageType app not supported');
				}
				else if(storageType=='online'){
					errorCallback('app.helper.storage.file.remove: storageType online not supported');
				}
				else{
					errorCallback('app.helper.storage.file.remove: wrong storageType Parameter');
				}
			},
			
			// check if file exist
			// storigeType Online not supported
			is: function(key, storageType, successCallback, errorCallback){
				
				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);

				var self = app.helper.storage;

				if(!storageType){
					// storygeType sql will be checked
					self.file.helper._isOnSql(key, function(result){
						if(result){
							successCallback(true);
						}
						else{
							self.file.helper._isFileOnFs(key, function(result){
								if(result){
									successCallback(true);
								}
								else{
									self.file.helper._isOnApp(key, function(result){
										if(result){
											successCallback(true);
										}
										else{
											successCallback(false);
										}
									}, errorCallback);
								}
							}, errorCallback);
						}
					}, errorCallback);
				}
				// storygeType fs will be selected
				else if(storageType=='sql'){
					self.file.helper._isOnSql(
						key,
						successCallback,
						errorCallback
					);
				}
				// storygeType fs will be selected
				else if(storageType=='fs'){
					self.file.helper._isFileOnFs(
						key,
						successCallback,
						errorCallback
					);
				}
				// storygeType fs will be selected
				else if(storageType=='app'){
					self.file.helper._isOnApp(
						key,
						successCallback,
						errorCallback
					);
				}
				else if(storageType=='online'){
					errorCallback('app.helper.storage.file.is: storageType online not supported');
				}
				else{
					errorCallback('app.helper.storage.file.is: wrong storageType Parameter');
				}
			},
			getLocationDataMap: function(fileKey,resultCallback){
				var self = app.helper.storage;
				var locMap = {location: null,type: null,data: null,status:false};
				var pathKey = fileKey;
				// remove '/' prefix
				if(pathKey.charAt(0)=="/"){
					pathKey = pathKey.substring(1);
				}

				// getManifestData
				var fileData = self._getManifestDataForKey(fileKey);
				// location data not found in manifest
				if (_.isEmpty(fileData)){
					resultCallback(locMap);
					return null;
				}

				switch(fileData.dataStorageType){
					case "fs":
						locMap.status = true;
						locMap.type = fileData.dataMimeType;
						locMap.location = self.file.helper._getStorageLocation("data")+pathKey;
						resultCallback(locMap);
					break;
					case "app":
						locMap.status = true;
						locMap.type = fileData.dataMimeType;
						locMap.location = pathKey;
						resultCallback(locMap);
					break;
					case "sql":
					case "online":
						self.file.get(
							fileKey,
							fileData.dataStorageType,
							function(d){
								locMap.status = true;
								locMap.type = fileData.dataMimeType;
								locMap.data = d;
								resultCallback(locMap);
							},
							function(e){
								resultCallback(locMap);
							}
						);						
					break;
					default:
						// unknown storageType
						resultCallback(locMap);
					break;
				}
				return null;
			},

			getFilePath: function(key, storageLocation, successCallback, errorCallback){

				((!successCallback)?successCallback=function(){}:null);
				((!errorCallback)?errorCallback=function(){}:null);

				var self = app.helper.storage;

				var fileOptions = {
					create: false,			// file will be created if not exists (if param is true)
					exclusive: false		// returns error if file exists (if param is true)
				};

				if(key.charAt(0)=="/"){
					key = key.substring(1);
				}

				if(!storageLocation || storageLocation=='fs'){
					self.file.helper._isFileOnFs(
						key,
						function(result){
							if(result){
								self.dataStorage.getFile(
									key,
									fileOptions,
									function(fileEntry){
										successCallback(fileEntry.nativeURL);
									},
									errorCallback
								);
							}
							else{
								successCallback(false);
							}
						},
						errorCallback
					);
				}
				else if(storageLocation=='sql' || storageLocation=='app' || storageLocation=='online'){
					errorCallback('app.helper.storage.file.getFilePath: storageLocation sql, app and online are not supported');
				}
				else{
					errorCallback;
				}
			},

			helper: {
				_getFromSql: function(key, successCallback, errorCallback){

					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);

					var self = app.helper.storage;

					self.database.transaction(
						function(tx){
							tx.executeSql(
								'SELECT * FROM files WHERE dataKey=? LIMIT 2;',
								[key],
								function(tx, result){
									if(result.rows.length==0){
										errorCallback('app.helper.storage.file.helper._getFromSql: not found');
									}
									else if(result.rows.length==1){
										successCallback(result.rows.item(0).fileContent);
									}
									else{
										errorCallback('app.helper.storage.file.helper._getFromSql: more than one file with same key in sql files table');
									}
								},
								errorCallback
							);
						}
					);
				},

				_getFromFs: function(key, successCallback, errorCallback){

					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);

					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}

					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							if(fileEntry.isFile){
								fileEntry.file(
									function(file){
										var reader = new FileReader();
										reader.readAsText(file);
										reader.onloadend = function(result){
											successCallback(this.result);
										};
										reader.onerror = errorCallback;
									},
									errorCallback
								);
							}
							else if(fileEntry.isDirectory){
								errorCallback('app.helper.storage.file.helper._getFromFs: parameter is directory, not file');
							}
							else{
								errorCallback('app.helper.storage.file.helper._getFromFs: unknown error');
							}
						},
						errorCallback

					);
				},

				_getFromApp: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}

					// workaround...
					loadXmlHttpFile(GLOBALS.includePath+key,successCallback,errorCallback);
					return;

					/*

					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							if(fileEntry.isFile){
								fileEntry.file(
									function(file){
										var reader = new FileReader();
										reader.readAsText(file);
										reader.onloadend = function(result){
											successCallback(this.result);
										};
										reader.onerror = function(error){
											errorCallback(error);
										};
									},
									errorCallback
								);
							}
							else if(fileEntry.isDirectory){
								errorCallback('app.helper.storage.file.helper._getFromApp: parameter is directory, not file');
							}
							else{
								errorCallback('app.helper.storage.file.helper._getFromApp: unknown error');
							}
						},
						errorCallback

					);

					*/
				},

				_getFromOnline: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					// do API-FILE REQUEST
					app.api.doRequestBinaryAsync(
						':file',
						{afid:key},
						function(contentBlob){
							self.file.helper._convertBlobToString(
								contentBlob,
								successCallback,
								errorCallback
							);
						},
						errorCallback
					);

					return;
				},

				_setToSql: function(key, fileContent, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;
					
					self.file.helper._isOnSql(
						key,
						function(result){
							self.file.helper._convertBlobToString(
								fileContent,
								function(convertResult){
									// file exist in sql, must be replaced
									if(result){
										self.database.transaction(
											function(tx){
												tx.executeSql(
													'UPDATE files SET fileContent=? WHERE dataKey=?;',
													[convertResult, key],
													function(tx, result){
														successCallback(true);
													},
													errorCallback
												);
											}
										);
									}
									// file does not exist in sql, must be created
									else{
										self.database.transaction(
											function(tx){
												tx.executeSql(
													'INSERT INTO files (dataKey, fileContent) VALUES (?, ?);',
													[key, convertResult],
													function(tx, result){
														successCallback(true);
													},
													errorCallback
												);
											}
										);
									}
								},
								errorCallback
							);
						},
						errorCallback
					);
				},

				_setToFs: function(key, fileContent, successCallback, errorCallback){					
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					var fileOptions = {
						create: true,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					// create Folder's if not exist
					var keyArray = key.split("/");
					if(keyArray.length>1){
						var fileName = keyArray.splice(-1)[0];
						var filePath = keyArray.join("/");

						self.file.helper.file._createDirRecursive(
							filePath,
							function(directoryEntry){
								directoryEntry.getFile(
									fileName,
									fileOptions,
									function(fileEntry){
										fileEntry.createWriter(
											function(fileWriter){
												// check, if 'fileContent' is blob
												if (_.isObject(fileContent)){
													fileWriter.write(fileContent);
												}else{
													var bb = new Blob([fileContent]);
													fileWriter.write(bb);
												}

												fileWriter.onwriteend = function(result){
													successCallback(true);
												};
												fileWriter.onerror = function(error){
													errorCallback(error);
												}
											},
											errorCallback
										);
									},
									errorCallback
								);
							},
							errorCallback
						);
					}
					else{
						self.dataStorage.getFile(
							key,
							fileOptions,
							function(fileEntry){
								fileEntry.createWriter(
									function(fileWriter){
										// check, if 'fileContent' is blob
										if (_.isObject(fileContent)){
											fileWriter.write(fileContent);
										}else{
											var bb = new Blob([fileContent]);
											fileWriter.write(bb);
										}

										fileWriter.onwriteend = function(result){
											successCallback(true);
										};
										fileWriter.onerror = function(error){
											errorCallback(error);
										}
									},
									errorCallback
								);
							},
							errorCallback

						);
					}
				},

				_removeFromSql: function(key, successCallback, errorCallback){					
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					self.file.helper._isOnSql(
						key, 
						function(result){
							if(result){
								self.database.transaction(
									function(tx){
										tx.executeSql(
											'DELETE FROM files WHERE dataKey=?;',
											[key],
											function(tx, result){
												successCallback(true);
											},
											errorCallback
										);
									}
								);
							}
							else{
								successCallback(false);
							}
						},
						errorCallback
					);
								
				},

				_removeFromFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};
					
					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							fileEntry.remove(
								function(tx, result){
									if (tx=='OK') {
										successCallback(true);
									};
								},
								errorCallback
							);
						},
						errorCallback
					);
				},

				_removeFileFromFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};
					
					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							fileEntry.remove(
								function(tx, result){
									if (tx=='OK') {
										successCallback(true);
									};
								},
								errorCallback
							);
						},
						errorCallback
					);
				},

				_removeDirFromFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var directoryOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};
					
					self.dataStorage.getDirectory(
						key,
						directoryOptions,
						function(directoryEntry){
							directoryEntry.remove(
								function(tx, result){
									if (tx=='OK') {
										successCallback(true);
									};
								},
								function(error){
									// error-code 1 -> file not found
									if(error.code==1){
										successCallback(false);
									}
									else{
										errorCallback(error);
									}
								}
							);
						},
						errorCallback
					);
				},

				_isOnSql: function(key, successCallback, errorCallback){					
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);					
					var self = app.helper.storage;
					
					self.database.transaction(
						function(tx){
							tx.executeSql(
								'SELECT * FROM files WHERE dataKey=? LIMIT 2;',
								[key],
								function(tx, result){
									if(result.rows.length==0){
										successCallback(false);
									}
									else if(result.rows.length==1){
										successCallback(true);
									}
									else{
										errorCallback('app.helper.storage.file.helper._isOnSql: more than one file with same key in sql files table');
									}
								},
								errorCallback
							);
						}
					);
				},

				// returns true if file or directory is found, else false
				_isOnFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);					
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};
					var directoryOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							successCallback(true);
						},
						function(error){
							// error-code 1 -> file not found
							if(error.code==1){
								successCallback(false);
							}
							// error-code 11 -> type missmatch
							else if(error.code==11){
								self.dataStorage.getDirectory(
									key,
									directoryOptions,
									function(fileEntry){
										successCallback(true);
									},
									function(error){
										// error-code 1 -> file not found
										if(error.code==1){
											successCallback(false);
										}
										else{
											errorCallback(error);
										}
									}
								);
							}
							else{
								errorCallback(error);
							}
						}

					);
				},

				// returns true / false if file in param key exist or not
				_isFileOnFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);					
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					self.dataStorage.getFile(
						key,
						fileOptions,
						function(fileEntry){
							successCallback(true);
						},
						function(error){
							// error-code 1 -> file not found
							if(error.code==1){
								successCallback(false);
							}
							// error-code 11 -> type missmatch
							else if(error.code==11){
								errorCallback('app.helper.storage.file.helper._isFileOnFs: found, but dir not file');
							}
							else{
								errorCallback(error);
							}
						}

					);
				},

				// returns true / false if directory in param key exist or not
				_isDirOnFs: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);					
					var self = app.helper.storage;

					if(key.charAt(0)=="/"){
						key = key.substring(1);
					}
					
					var directoryOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					window.resolveLocalFileSystemURL(
						self.file.helper._getStorageLocation('data'),
						function(directoryEntry){
							directoryEntry.getDirectory(
								key,
								directoryOptions,
								function(fileEntry){
									successCallback(true);
								},
								function(error){
									// error-code 1 -> file not found
									if(error.code==1){
										successCallback(false);
									}
									// error-code 11 -> type missmatch
									else if(error.code==11){
										errorCallback('app.helper.storage.file.helper._isDirOnFs: found, but file not dir');
									}
									else{
										errorCallback(error);
									}
								}

							);
						},
						function(error){
							console.error('directoryEntry ' + error);
						}
					);
				},

				_isOnApp: function(key, successCallback, errorCallback){
					((!successCallback)?successCallback=function(){}:null);
					((!errorCallback)?errorCallback=function(){}:null);					
					var self = app.helper.storage;

					// TO DO workaround 4 android bug
					self.file.helper._getFromApp(
						key,
						function(){
							successCallback(true);
						},
						function(){
							successCallback(false);
						}
					);

					/*
					var fileOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};
					var directoryOptions = {
						create: false,			// file will be created if not exists (if param is true)
						exclusive: false		// returns error if file exists (if param is true)
					};

					window.resolveLocalFileSystemURL(
						//self.file.helper._getStorageLocation('application'),
						cordova.file.applicationDirectory,
						function(directoryEntry){
							directoryEntry.getFile(
								key,
								fileOptions,
								function(fileEntry){
									successCallback(true);
								},
								function(error){
									// error-code 1 -> file not found
									if(error.code==1){
										successCallback(false);
									}
									// error-code 11 -> type missmatch
									else if(error.code==11){
										directoryEntry.getDirectory(
											key,
											directoryOptions,
											function(fileEntry){
												successCallback(true);
											},
											function(error){
												// error-code 1 -> file not found
												if(error.code==1){
													successCallback(false);
												}
												else{
													errorCallback(error);
												}
											}
										);
									}
									else{
										errorCallback(error);
									}
								}

							);
						},
						function(error){
							// TO DO
							if(cordova.platformId == 'android'){
								successCallback(false);
								console.info('app.helper.storage.file.helper._isOnApp: android bug, no app storage access possible. return successCallback(false) for '+key);
							}
							else{
								errorCallback(error);
							}
						}
					);
					*/
				},

				file: {
					// creates one directorys on the key string
					// returns a directoryEntry-Object on Success
					_createDir: function(key, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);
						var self = app.helper.storage;

						if(key.charAt(0)=="/"){
							key = key.substring(1);
						}

						window.resolveLocalFileSystemURL(
							self.file.helper._getStorageLocation('data'),
							//cordova.file.dataDirectory,
							function(directoryEntry){
								directoryEntry.getDirectory(
									key,
									{create:true},
									function(newDirectoryEntry){
										successCallback(newDirectoryEntry);
									},
									errorCallback

								);
							},
							errorCallback
						);
					},

					// creates recursive directorys on the key string like aaa/bbb/ccc
					// returns a directoryEntry-Object on Success
					_createDirRecursive: function(key, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);
						var self = app.helper.storage;
						var successCallback = successCallback;

						if(key.charAt(0)=="/"){
							key = key.substring(1);
						}

						keyArray = key.split("/").reverse();

						var localCreateDir = function(key, directoryEntry){
							directoryEntry.getDirectory(
								key,
								{create:true, exclusive : false},
								localSuccessCallback,
								localErrorCallback
							);
					    };

					    var localSuccessCallback = function(directoryEntry){
					        if(keyArray.length > 0){
					            localCreateDir(keyArray.pop(), directoryEntry);
					        }
					        else{
					            successCallback(directoryEntry);
					        }
					    };

					    var localErrorCallback = function(){
					        errorCallback;
					    };

				        window.resolveLocalFileSystemURL(
							self.file.helper._getStorageLocation('data'),
							function(directoryEntry){
				    			localCreateDir(keyArray.pop(), directoryEntry);
							},
							localErrorCallback
						);
					},

					_removeDir: function(key, successCallback, errorCallback){
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);
						var self = app.helper.storage;

						if(key.charAt(0)=="/"){
							key = key.substring(1);
						}
						
						var directoryOptions = {
							create: false,			// file will be created if not exists (if param is true)
							exclusive: false		// returns error if file exists (if param is true)
						};
						
						self.dataStorage.getDirectory(
							key,
							directoryOptions,
							function(directoryEntry){
								directoryEntry.remove(
									function(tx, result){
										if (tx=='OK') {
											successCallback(true);
										};
									},
									function(error){
										// error-code 1 -> file not found
										if(error.code==1){
											successCallback(false);
										}
										else{
											errorCallback(error);
										}
									}
								);
							},
							errorCallback
						);
					},

					_removeDirRecursive: function(key, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);
						var self = app.helper.storage;

						if(key.charAt(0)=="/"){
							key = key.substring(1);
						}

						window.resolveLocalFileSystemURL(
							self.file.helper._getStorageLocation('data'),
							function(directoryEntry){
								directoryEntry.getDirectory(
									key,
									{create:false},
									function(newDirectoryEntry){
										newDirectoryEntry.removeRecursively(
											function(tx, result){
												if (tx=='OK') {
													successCallback(true);
												};
											},
											errorCallback
										);
									},
									errorCallback
								);
							},
							errorCallback
						);
					},

					_listDirContent: function(key, successCallback, errorCallback){						
						((!key)?key='':null);
						var self = app.helper.storage;

						if(key.charAt(0)=="/"){
							key = key.substring(1);
						}

						keyArray = key.split("/");
						if(keyArray.length>=2){
							var j = 0;
							var runA = function(directoryEntry){
								directoryEntry.getDirectory(
									keyArray[j],
									{create:false},
									function(newDirectoryEntry1){
										j++;
										runB(newDirectoryEntry1);
									},
									function(e){
										console.error(e);
									}
								);
							};
							var runB = function(directoryEntry){
								if(j<(keyArray.length)){
									runA(directoryEntry);
								}
								else{
									var runC = function(entries){
										var i = 0;
										while(i<entries.length){
											if(entries[i].isDirectory){
												getEntrys(entries[i]);
											}
											console.log(entries[i]);
											i++;
										}
									}
									
									var getEntrys = function(localDirectoryEntry){
										var entries = Array();
										var dirReader = localDirectoryEntry.createReader();
										var read = function(){
											dirReader.readEntries(
												function(results){
													if(!results.length){
														runC(entries);
													}
													else{
														var i = 0;
														while(i<results.length){
															entries.push(results[i]);
															i++;
														}
														read();
													}
												},
												function(e){
													console.error(e);
												}
											);
										}
										read();
									};

									getEntrys(directoryEntry);
								}
							};
							runA(self.dataStorage);
						}
						else if (keyArray.length==1){
							self.dataStorage.getDirectory(
								key,
								{create:false},
								function(directoryEntry){
									var runC = function(entries){
										var i = 0;
										while(i<entries.length){
											if(entries[i].isDirectory){
												getEntrys(entries[i]);
											}
											console.log(entries[i]);
											i++;
										}
									}
									
									var getEntrys = function(directoryEntry){
										var entries = Array();
										var dirReader = directoryEntry.createReader();
										var read = function(){
											dirReader.readEntries(
												function(results){
													if(!results.length){
														runC(entries);
													}
													else{
														var i = 0;
														while(i<results.length){
															entries.push(results[i]);
															i++;
														}
														read();
													}
												},
												function(e){
													console.error(e);
												}
											);
										}
										read();
									};

									getEntrys(directoryEntry);
								},
								function(e){
									console.error(e);
								}
							);
						}
						else{
							var allEntries = [];
							var entries = [];
							var getEntrys = function(directoryEntry){
								var dirReader = directoryEntry.createReader();
								var read = function(){
									dirReader.readEntries(
										function(results) {
											//console.log(results);
											if(!results.length){
												var i = 0;
												while(i<entries.length){
													if(entries[i].isDirectory){
														//console.log(entries[i].name);
														//getEntrys(entries[i]);
													}
													console.log(entries[i]);
													i++;
												}
											}
											else{
												var i = 0;
												while(i<results.length){
													entries.push(results[i]);
													i++;
												}
												read();
											}
										},
										function(e){
											console.error(e);
										}
									);
								}
								var i = 0;
								while(i<entries.length){
									allEntries.push(entries[i]);
									i++;
								}
								entries = [];
								read();
							};
							getEntrys(self.dataStorage);
						}
					}
				},

				sql: {
					_createTable: function(table, collumns, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);						
						var self = app.helper.storage;

						self.database.transaction(
							function(tx){
								tx.executeSql(
									'CREATE TABLE IF NOT EXISTS ' + table + ' (' + collumns + ');',
									[],
									function(tx, result){
										// SUCCESS-HANDLING
										successCallback(result);
										console.log(tx, result);
									},
									errorCallback
								);
							}
						);
					},

					_dropTable: function(table, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);
						var self = app.helper.storage;

						self.database.transaction(
							function(tx){
								tx.executeSql(
									'DROP TABLE ' + table,
									[],
									function(tx, result){
										// SUCCESS-HANDLING
										successCallback(result);
										console.log(tx, result);
									},
									errorCallback
								);
							}
						);
					},
					
					_isTable: function(table, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);						
						var self = app.helper.storage;

						self.database.transaction(
							function(tx){  
								tx.executeSql(
									'SELECT name FROM sqlite_master WHERE type=\'table\' AND name=?;',
									[table],
									function(tx, result){
										if(result.rows.length===0 || result.rows.length>1){
											// SUCCESS-HANDLING return false
											successCallback(false);
											console.log(false);
										}
										else{
											// SUCCESS-HANDLING return true
											successCallback(true);
											console.log(true);
										}
									},
									errorCallback
								);
							}
						);
					},

					_isEntity: function(key, table, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);						
						var self = app.helper.storage;

						self.database.transaction(
							function(tx){  
								tx.executeSql(
									'SELECT * FROM ' + table + ' WHERE dataKey=? LIMIT 1;',
									[key],
									function(tx, result){
										if(result.rows.length===0){
											successCallback(false);
										}
										else{
											successCallback(true);
										}
									},
									errorCallback
								);
							}
						);
					},

					_run: function(sqlString, successCallback, errorCallback){						
						((!successCallback)?successCallback=function(){}:null);
						((!errorCallback)?errorCallback=function(){}:null);						
						var self = app.helper.storage;

						self.database.transaction(
							function(tx){
								tx.executeSql(
									sqlString,
									[],
									function(tx, result){
										// SUCCESS-HANDLING
										for (var i = 0; i < result.rows.length; i++){
											console.log(result.rows.item(i));
										};
										//successCallback(result);
										console.log(tx, result);
									},
									errorCallback
								);
							}
						);
					}
				},

				// PERSISTENT LOCATIONS TO READ/WRITE DATA
				_getStorageLocation: function(locationType){
					var _application = function(){
						return cordova.file.applicationDirectory;
					};
					var _data = function(){
						var dynLocation = app.settings.get('dataStorage');
						switch (dynLocation){
							case 'applicationDirectory':
								return cordova.file.applicationDirectory;
							break;
							case 'applicationStorageDirectory':
								return cordova.file.applicationStorageDirectory;
							break;
							case 'cacheDirectory':
								return cordova.file.cacheDirectory;
							break;
							case 'externalApplicationStorageDirectory':
								return cordova.file.externalApplicationStorageDirectory;
							break;
							case 'dataDirectory':
							default:
								return cordova.file.dataDirectory;
							break;
						}
					};

					switch (locationType){
						case 'app':
						case 'application':
							return _application();
						break;
						case 'data':
						case 'storage':
						default:
							return _data();
						break;
					}
				},

				_convertBlobToString: function(contentBlob, successCallback, errorCallback){
					
					// convert image to base64
					if(contentBlob instanceof Blob && contentBlob.type.split("/")[0]=='image'){
						var reader = new FileReader();
						reader.readAsBinaryString(contentBlob);
						reader.onloadend = function(result){
							successCallback(btoa(this.result));
						};
						reader.onerror = errorCallback;
					}
					// convert all other to string
					else if(contentBlob instanceof Blob){
						var reader = new FileReader();
						reader.readAsText(contentBlob);
						reader.onloadend = function(result){
							successCallback(this.result);
						};
						reader.onerror = errorCallback;
					}
					else{
						errorCallback('app.helper.storage.file.helper._convertBlobToString: first param is not a blob');
					}
				}
			}
		},

		// setup the storage (database)
		_setup: function(successCallback, errorCallback){			
			((!successCallback)?successCallback=function(){}:null);
			((!errorCallback)?errorCallback=function(){}:null);			
			var self = app.helper.storage;

			// validate database
			if (_.isNull(self.database)){
				return errorCallback("storageHelper.setup: database.notready");
			}

			// DB SETUP
			var _dbSetupQueries = [
				{q: 'CREATE TABLE IF NOT EXISTS settings (dataKey TEXT NOT NULL, dataValue TEXT NOT NULL);', p: []},
				{q: 'CREATE TABLE IF NOT EXISTS files (dataKey TEXT NOT NULL, fileContent TEXT NOT NULL);', p: []}
			];

			var _dbSetup = function(tx,qObj){
				// break condition
				if (_.isNull(qObj) || _.isEmpty(qObj)){
					// change here, if other 'setups' are required...
					successCallback();
					return;
				}
				// exec current query
				tx.executeSql(
					qObj.q,
					qObj.p,
					function(newTx,res){
						// start with next job
						_dbSetup(tx,_dbSetupQueries.pop());
					},
					errorCallback
				);
			};

			self.database.transaction(
				function(cTx){
					// start with first query
					_dbSetup(cTx,_dbSetupQueries.pop());
				}
			);
		},
		_getManifestDataForKey: function(refKey){
			var cManifest = app.settings.get("MANIFEST");
			var mapData = {};

			// add '/' prefix
			if(refKey.charAt(0) != "/"){
				refKey = "/"+refKey;
			}

			_.forEach(cManifest.dataFiles,function(fData){
				if (fData.dataKey == refKey){
					mapData = fData;
					return;
				}
			});
			return mapData;
		},
		_getManifestDataForMap: function(refMap){
			var cManifest = app.settings.get("MANIFEST");
			var mapData = {};

			_.forEach(cManifest.dataFiles,function(fData){
				if (fData.dataMapping == refMap){
					mapData = fData;
					return;
				}
			});
			return mapData;
		},

		// the general init function
		_init: function(){

		},

		// initReady is called by 'app'-class
		_initReady: function(){
			var self = app.helper.storage;

			// preserve ready status for FileSystem & database
			_.preserveReady(self);	// for FS
			_.preserveReady(self);	// for DB

			// load filesystem - async
			window.resolveLocalFileSystemURL(
				self.file.helper._getStorageLocation('data'),
				function(directoryEntry){
					self.dataStorage = directoryEntry;
					_.setReady(self);
				},
				function(error){
					eHandler.exception(error);
				}
			);

			// validate function
			if (_.isUndefined(window.openDatabase)){
				eHandler.exception("Unable to load database!",'storageHelper');
				return false;
			}

			// load database - sync
			self.database = window.openDatabase(app.settings.get('sqlDatabaseName'), '1.0', app.settings.get('sqlDatabaseName'), app.settings.get('sqlDatabaseSize'));
			if (_.isObject(self.database)){
				_.setReady(self);
			}
		}
	};
	app.helper.storage._init();
})(window.app);