(function(app,windowModel){
	app.system = {
		// need this for eListener
        _className: 'system',

        pinnedProductsList: [],

        analytics: {
        	_map: {},
        	init: function(successCallback, errorCallback){
        		var self = app.system;

        		// return, if anaytics is disabled
        		if ((!app.settings.get('analytics.status')) || _.isUndefined(analytics)){
        			successCallback();
        			return null;
        		}        		

				// the analytics enable function
				var _enableAnalytics = function(){
					var trackingID = app.settings.get('analytics.id');

					if (_.isEmpty(trackingID) || _.isNull(trackingID)){
						console.warn("analytics.init: missing or empty 'trackingID'!");
						return false;
					}

					// general init
					analytics.startTrackerWithId(trackingID);

					// append the callback function
					app.nav.register.pageCallback("changePage",function(pParams){
						var langPrefix = app.settings.get("LANGUAGE");

						// do not track unknown trackings
						if (_.isUndefined(app.system.analytics._map[pParams.pageName])){
							console.warn("analytics.ignore: "+pParams.pageName);
							return false;
						}

						var trackPageName = app.system.analytics._map[pParams.pageName];

						// check for pageId
						if (!_.isUndefined(pParams['pageParams'].id)){
							var _trackingPageId = langPrefix+"_"+trackPageName+"_"+pParams['pageParams'].id;
						}else{
							var _trackingPageId = langPrefix+"_"+trackPageName;
						}

						// track view by analytics
						analytics.trackView(_trackingPageId);
						console.log("analytics.track: "+_trackingPageId);
					});

					successCallback();
				};

				// the mapCheck function
				var _requestMap = function(){
					// ga mapping filename
					var mapKey = app.settings.get("FMAP.analytics");

					// getFileContent
                    app.helper.storage.file.get(
                        mapKey,
                        ':manifest:',
                        function(d){
                        	self.analytics._map = parse(d);
                        	_enableAnalytics();
                        },
                        errorCallback
                    );
				};

				// start analytics mapping
				_requestMap();
        	}
        },
		cartCountValue: function(setVar){
			if (_.isUndefined(setVar)){
				var cHeader = models("#header");
				if (cHeader){
					return cHeader.data.cartCountValue();
				}
			}else{
				var cHeader = models("#header");
				if (cHeader){
					cHeader.data.cartCountValue(setVar);
				}
			}
			return null;
		},
		logout: function(quiet){
			((!quiet)?quiet=false:null);
			analytics.trackEvent('Settings', 'logout', null, null);
			var cLang = app.settings.get('LANGUAGE');
			if(device.platform.toLowerCase()=='android'){
				// adjust android logout event
				if(cLang=='de'){
					Adjust.trackEvent('3h3a9h');
				}
				else if(cLang=='en'){
					Adjust.trackEvent('mhwx7y');
				}
				else if(cLang=='es'){
					Adjust.trackEvent('3vr5yl');
				}
				else if(cLang=='fr'){
					Adjust.trackEvent('kuro8q');
				}
				else if(cLang=='it'){
					Adjust.trackEvent('j4zhm2');
				}
				else if(cLang=='nl'){
					Adjust.trackEvent('tgn3fz');
				}
				else if(cLang=='pl'){
					Adjust.trackEvent('7j0igc');
				}
			}
			else if(device.platform.toLowerCase()=='ios'){
				// adjust ios logout event
				if(cLang=='de'){
					Adjust.trackEvent('bigmj9');
				}
				else if(cLang=='en'){
					Adjust.trackEvent('5bhu2m');
				}
				else if(cLang=='es'){
					Adjust.trackEvent('fh6d6h');
				}
				else if(cLang=='fr'){
					Adjust.trackEvent('oqtxpb');
				}
				else if(cLang=='it'){
					Adjust.trackEvent('vy9cw1');
				}
				else if(cLang=='nl'){
					Adjust.trackEvent('52oj5c');
				}
				else if(cLang=='pl'){
					Adjust.trackEvent('o5lr2n');
				}
			}
			app.settings.set("api.sessionToken","",true);
			app.settings.set("api.rememberToken","",true);
			app.settings.set("api.userLoggedIn",false,true);
			app.settings.set("api.userName","",true);
			facebookConnectPlugin.logout(
				function(succ){
					console.log('FB-Plugin Logout ok');
				},
				function(){
					console.error('FB-Plugin Logout error');
				}
			);
			if(!quiet){
				app.nav.alert.show($.t('general.messages.logoutSuccessfull'));
				app.nav.loadPage('#startup', function(){
					app.nav.history.clear();
				});
			}
		},
		push: {
			// attention! sC & eC runs over register_ready function because of differences between ios and android!
			init: function(){
				document.addEventListener("urbanairship.registration", function(event){
					if(event.error){
						console.error('there was an error registering for push notifications');
					}else{
						app.settings.set("pn.deviceToken", event.pushID, true);
						app.api.doAPIRequestAsync(
							':apiPost',
							'device_token_registrations',
							{"device_token_registration":{"device_token":event.pushID}},
							'post',
							function(result){
								if(result.result===true){
									console.info('PUSH.init - api: send ok');
								}
								else{
									console.error('PUSH.init - api: send error',result);
								}
							},
							function(err){
								console.error('PUSH.init - api: send error',err);
							}
						);
						console.info('PUSH.init - device token: '+event.pushID);
					} 
				}, false);

				document.addEventListener("urbanairship.push", app.system.push.handler, false);
			},
			register: function(sC,eC){
				var channels = new Array();
				if(app.settings.get("pn.channels.reminders")){
					channels.push(app.settings.get('LANGUAGE')+'_reminders');
				}
				if(app.settings.get("pn.channels.inspirations")){
					channels.push(app.settings.get('LANGUAGE')+'_inspirations');
				}
				if(app.settings.get("pn.channels.highlights")){
					channels.push(app.settings.get('LANGUAGE')+'_highlights');
				}

				console.info("PUSH.register - channels: ",channels);

				if(channels.length > 0){
					PushNotification.setTags(channels, function(){
						PushNotification.isPushEnabled(function(isPushEnabled){
							if(!isPushEnabled){
								PushNotification.enablePush(function(){
									console.info('PUSH.register - status: enabled');

									// for iOS only
									if(device.platform.toLowerCase()=='ios'){
										PushNotification.registerForNotificationTypes(PushNotification.notificationType.sound | PushNotification.notificationType.alert | PushNotification.notificationType.badge, function(){
											console.info('PUSH.register - badge types set (ios only)');
											sC();
										});
									}else{
										sC();
									}
								});
							}
							else{
								sC();
							}
						});
					});
				}else{
					PushNotification.isPushEnabled(function(isPushEnabled){
						if(isPushEnabled){
							PushNotification.disablePush(function(){
								console.log('PUSH.register - status: disabled');
								sC();
							});
						}
						else{
							sC();
						}
					});
				}
			},
			handler: function(result){
				if(app.settings.get("pn.channels.reminders") || app.settings.get("pn.channels.inspirations") || app.settings.get("pn.channels.highlights")){
					PushNotification.isPushEnabled(function(result2){
						if(result2){
							console.info("PUSH.handler - show message");
							app.nav.alert.show(result.message, 'info');
						}else{
							console.info("PUSH.handler - hide message (push)");
						}
					});
				}else{
					console.info("PUSH.handler - hide message (settings)");
				}
			}
		},
		pinProduct: function(cObj, productId){
			var self = this;
			if(!self.checkProductOnPinnedProductsList(productId)){
				app.api.doAPIRequestAsync(
					':apiPost',
					'product_pins',
					{'product_pin':{'product_id':productId}},
					'post',
					function(result){
						if(result.result===true){
							analytics.trackEvent('Pins', 'pinned', null, productId);
							var cLang = app.settings.get('LANGUAGE');
							if(device.platform.toLowerCase()=='android'){
								// adjust android pin event
								if(cLang=='de'){
									Adjust.trackEvent('8xthp3');
								}
								else if(cLang=='en'){
									Adjust.trackEvent('kbvj7b');
								}
								else if(cLang=='es'){
									Adjust.trackEvent('59zn81');
								}
								else if(cLang=='fr'){
									Adjust.trackEvent('wo3ttg');
								}
								else if(cLang=='it'){
									Adjust.trackEvent('gic5sq');
								}
								else if(cLang=='nl'){
									Adjust.trackEvent('bmvott');
								}
								else if(cLang=='pl'){
									Adjust.trackEvent('4ci6rr');
								}
							}
							else if(device.platform.toLowerCase()=='ios'){
								// adjust ios pin event
								if(cLang=='de'){
									Adjust.trackEvent('rt15nu');
								}
								else if(cLang=='en'){
									Adjust.trackEvent('yqb6fg');
								}
								else if(cLang=='es'){
									Adjust.trackEvent('zebz2g');
								}
								else if(cLang=='fr'){
									Adjust.trackEvent('y093jd');
								}
								else if(cLang=='it'){
									Adjust.trackEvent('pavpz7');
								}
								else if(cLang=='nl'){
									Adjust.trackEvent('369u8u');
								}
								else if(cLang=='pl'){
									Adjust.trackEvent('t91m1n');
								}
							}
							app.nav.alert.show($.t('pinboard.messages.successfullyPinned'));
							$(cObj).addClass('active');
							$(cObj).attr('onclick','app.system.removePinnedProduct(this, '+productId+');');
							self.pinnedProductsList.push(result.data);
						}
						else{
							app.nav.alert.show($.t('pinboard.messages.errorUnknownError'));
						}
					},
					function(err){
						app.nav.alert.show($.t('pinboard.messages.errorUnknownError'), 'error');
					}
				);
			}
			else{
				app.nav.alert.show($.t('pinboard.messages.productAlreadyInList'));
				$(cObj).addClass('active');
				$(cObj).attr('onclick','app.system.removePinnedProduct(this, '+productId+');');
			}
		},
		removePinnedProduct: function(cObj, productId, removeDomObj, siteIsPinboard){
			((!removeDomObj)?removeDomObj=false:null);
			((!siteIsPinboard)?siteIsPinboard=false:null);
			var self = this;
			app.api.doAPIRequestAsync(
				':apiDelete',
				'product_pins/'+productId,
				{},
				'delete',
				function(result){
					if(result.result===true){
						analytics.trackEvent('Pins', 'unpinned', null, productId);
						var cLang = app.settings.get('LANGUAGE');
						if(device.platform.toLowerCase()=='android'){
							// adjust android unpin event
							if(cLang=='de'){
								Adjust.trackEvent('3ab0iy');
							}
							else if(cLang=='en'){
								Adjust.trackEvent('2wthww');
							}
							else if(cLang=='es'){
								Adjust.trackEvent('kxgt37');
							}
							else if(cLang=='fr'){
								Adjust.trackEvent('8x1kne');
							}
							else if(cLang=='it'){
								Adjust.trackEvent('j70hdo');
							}
							else if(cLang=='nl'){
								Adjust.trackEvent('gsiokq');
							}
							else if(cLang=='pl'){
								Adjust.trackEvent('stin1u');
							}
						}
						else if(device.platform.toLowerCase()=='ios'){
							// adjust ios unpin event
							if(cLang=='de'){
								Adjust.trackEvent('83a3tx');
							}
							else if(cLang=='en'){
								Adjust.trackEvent('8n10yp');
							}
							else if(cLang=='es'){
								Adjust.trackEvent('wz7p5v');
							}
							else if(cLang=='fr'){
								Adjust.trackEvent('79cea0');
							}
							else if(cLang=='it'){
								Adjust.trackEvent('epjeyv');
							}
							else if(cLang=='nl'){
								Adjust.trackEvent('kcmnky');
							}
							else if(cLang=='pl'){
								Adjust.trackEvent('i1fnyi');
							}
						}
						$(cObj).removeClass('active');
						$(cObj).attr('onclick','app.system.pinProduct(this, '+productId+');');
						app.nav.alert.show($.t('pinboard.messages.successfullyUnpinned'));
						var arrayIdToDelete;
						_.forEach(app.system.pinnedProductsList, function(cElement, cKey){
							if(cElement.id == productId){
								arrayIdToDelete = cKey;
							}
						});
						app.system.pinnedProductsList.splice(arrayIdToDelete, 1);
						if(removeDomObj){
							$(cObj).parent().parent().parent().fadeOut(
								500,
								function(){
									$(this).remove();
									if(siteIsPinboard && self.pinnedProductsList.length==0){
										models('#pinboard').page.getNoResults().removeClass("hidden");
									}
								}
							);
						}
					}
					else{
						$(cObj).removeClass('active');
						$(cObj).attr('onclick','app.system.pinProduct(this, '+productId+');');
						app.nav.alert.show($.t('pinboard.messages.productNoMoreInList'), 'error');
					}
				},
				function(err){
					app.nav.alert.show($.t('pinboard.messages.errorUnknownError'));
					console.error(err);
				}
			);
		},
		setPinnedProductsList: function(dataArray){
			var self = this;
			self.pinnedProductsList = dataArray;
		},
		refreshPinnedProductsList: function(sC,eC){
			var self = this;
			app.api.doAPIRequestAsync(
				':api',
				'product_pins',
				{},
				'get',
				function(result){
					self.pinnedProductsList = result.data;
					sC();
				},
				function(err){
					//ERROR
					eC();
				}
			);
		},
		checkProductOnPinnedProductsList: function(cProductId){
			var self = this;

			var wasFound = false;

			_(self.pinnedProductsList).forEach(function(cElement){
				if(cElement.id==cProductId){
					wasFound = true;
				}
			});

			return wasFound;
		},
		share: function(productId, productImageUri, productDescription){
			if (!app.nav.page.is.loadedPage("#share")){
				app.nav.loadPage("#share",{pageLvl: 4, noHistory: true, restoreModel: true},function(){
					models('#share')._def.productId = productId;
					models('#share')._def.productImageUri = productImageUri;
					models('#share')._def.productDescription = productDescription;
					$('.btn-back:first').attr({'onclick':'models("#share").closeShare();'});
					app.nav.showLevel(4);
				});
			}
			else{
				models('#share')._def.productId = productId;
				models('#share')._def.productImageUri = productImageUri;
				models('#share')._def.productDescription = productDescription;
					$('.btn-back:first').attr({'onclick':'models("#share").closeShare();'});
				app.nav.showLevel(4);
			}
		},
		formatAmount: function(cents, currency){
			currency = currency.toLowerCase();
			var targetCurrency = app.settings.get("CURRENCY").toLowerCase();
			var cDate = new Date().getTime();

			var convertCurrency = function(){
				if(currency == 'eur'){
					if(targetCurrency == 'eur'){
						return (cents);
					}
					else if(targetCurrency == 'pln'){
						return (cents * app.settings.get("CURRENCY_PLN"));
					}
					else if(targetCurrency == 'gbp'){
						return (cents * app.settings.get("CURRENCY_GBP"));
					}
					else{
						return null;
					}
				}
				else if(currency == 'pln'){
					if(targetCurrency == 'eur'){
						return (cents / app.settings.get("CURRENCY_PLN"));
					}
					else if(targetCurrency == 'pln'){
						return (cents);
					}
					else if(targetCurrency == 'gbp'){
						return (cents / app.settings.get("CURRENCY_PLN") * app.settings.get("CURRENCY_GBP"));
					}
					else{
						return null;
					}
				}
				else if(currency == 'gbp'){
					if(targetCurrency == 'eur'){
						return (cents / app.settings.get("CURRENCY_GBP"));
					}
					else if(targetCurrency == 'pln'){
						return (cents / app.settings.get("CURRENCY_GBP") * app.settings.get("CURRENCY_PLN"));
					}
					else if(targetCurrency == 'gbp'){
						return (cents);
					}
					else{
						return null;
					}
				}
				else{
					return null;
				}
			}

			var calculateAmount = function(cACents){
				var amount = parseFloat(cACents / 100.0),
				precision = isNaN(precision = Math.abs(precision)) ? 2 : precision,
				delimiter = delimiter == undefined ? "," : delimiter,
				separator = separator == undefined ? "." : separator,
				sign = amount < 0 ? "-" : "",
				i = parseInt(amount = Math.abs(+amount || 0).toFixed(precision)) + "",
				j = (j = i.length) > 3 ? j % 3 : 0;
				return sign + (j ? i.substr(0, j) + separator : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + separator) + (precision ? delimiter + Math.abs(amount - i).toFixed(precision).slice(2) : "");
			}

			if(targetCurrency == 'eur'){
				return calculateAmount(convertCurrency()) + " €";
			}
			else if(targetCurrency == 'gbp'){
				return calculateAmount(convertCurrency()) + " £";
			}
			else if(targetCurrency == 'pln'){
				return calculateAmount(convertCurrency()) + " zł";
			}
			else{
				return null;
			}
		},
		checkDataTimeouts: function(sC, eC){
			((!sC)?sC=function(){}:null);
			((!eC)?eC=function(){}:null);

			var cDate = new Date().getTime();
			var lastCurrencyUpdate = app.settings.get("CURRENCY_LAST_UPDATE");
			
			// check if undefinded or 24h gone
			if((_.isUndefined(lastCurrencyUpdate)) || ((cDate-lastCurrencyUpdate) > 86400000)){
				app.api.doAPIRequestAsync(
					':apiGetCurrency',
					'',
					{},
					'get',
					function(result){
						if(result.result===true){
							app.settings.set("CURRENCY_LAST_UPDATE", cDate, true);
							app.settings.set("CURRENCY_GBP", result.data.GBP, true);
							app.settings.set("CURRENCY_PLN", result.data.PLN, true);
							app.settings.set("CURRENCY_EUR", 1, true);
							sC();
						}
						else{
							eC(result);
						}
					},
					function(result){
						eC(result);
					}
				);
			}
			else{
				sC();
			}
		},
		_init: function(){
			var self = this;
			// register READY
			eListener.register(["settings"],app.system._initReady);
		},
		_initReady: function(){
			// reference to global - unable to load through eListener ('this' is eListener)
			var self = app.system;

			// preserve ready status for FileSystem & dataBase
            _.preserveReady(self);  // for GA
            //_.preserveReady(self);  // for LANG

			// bind 'callback' function for googleAnalytics
			self.analytics.init(
				function(){
					_.setReady(self);
				},
				function(){
					// also set ready on error request!
					_.setReady(self);
				}
			);

			// bind 'callback' function for language
			/*
			self.initLanguage(function(){
				_.setReady(self);
			});
			*/
		}
	};
	app.system._init();
})(window.app,model);