/*************************
 * thirdparty extensions *
 *************************/

// global model func
var models = function(mIdent){
    if (_.isUndefined(app.nav) || _.isEmpty(app.nav)){
        return {};
    }

    if (_.isNumber(mIdent)){
        return {};
    }
    return app.nav.model.get(mIdent);
};

// global pages func
var pages = function(mIdent){
    return $(".page[data-ident='"+mIdent+"']:first");
};

// jQuery helper function for 'onload'-callback on images
var imgOnloadTransition = function(imgObj,targetContainer){
    var cImg = $(imgObj);

    // this will wait for 'preloaded' data
    if (!_.isUndefined(cImg.attr("data-load-seq"))){
        var loadSeq = parseInt(cImg.attr("data-load-seq"));

        // find closest page
        var pageIdent = cImg.closest(".page").attr("data-ident");
        var pTransObj = models(pageIdent).func.get("img-onload-transition");

        // register fadeFunc
        pTransObj.cbfunc[loadSeq] = function(){cImg.closest(targetContainer).addClass("transition-fade-box-in").removeClass("transition-fade-box-out");};

        // fire all events
        pTransObj.cbhandle(loadSeq);
    }else{
        cImg.closest(targetContainer).addClass("transition-fade-box-in").removeClass("transition-fade-box-out");
    }
    return true;
};

// global eventListener
var eListener = {
    _readyTypes: {},
    _readyFunctions: [],
    register: function(targetTypes,callbackFunc){
        if (!_.isArray(targetTypes)){
            targetTypes = [targetTypes];
        }
        console.info("eListener.register",targetTypes);

        var tmp = {
            status: false,
            targets: targetTypes,
            callback: callbackFunc
        };
        this._readyFunctions.push(tmp);
    },
    // calls the ready func for the targetType
    ready: function(targetType){
        console.info("eListener.ready",targetType);
        this._readyTypes[targetType] = true;
        this._fireCallbacks(targetType);
    },
    _fireCallbacks: function(cType){
        var self = this;
        _.forEach(self._readyFunctions,function(readyFunc,rPos){
            if (!readyFunc.status){
                if (self._checkReadyTargets(readyFunc.targets)){
                    self._readyFunctions[rPos].status = true;
                    if (_.isFunction(readyFunc.callback)){
                        console.info("eListener.fire["+rPos+"] - '"+cType+"'",readyFunc.callback);
                        readyFunc.callback();
                    }else{
                        console.error("eListener.fire["+rPos+"] is not a callbackfunction: ",readyFunc.callback);
                    }
                }
            }
        });
    },
    _isReadyType: function(readyType){
        if ((!_.isUndefined(this._readyTypes[readyType])) && this._readyTypes[readyType] == true){
            return true;
        }
        return false;
    },
    _checkReadyTargets: function(allTargets){
        var self = this;
        var fullRes = true;
        _.forEach(allTargets,function(readyTarget){
            if (!self._isReadyType(readyTarget)){
                fullRes = false;
            }
        });
        return fullRes;
    }
};

// global error handler
var eHandler = {
    'error': function(cObj,eResult){
        var eClass = cObj;
        if (_.isObject(cObj) && (!_.isUndefined(cObj._className))){
            eClass = cObj._className;
        }
        console.error("eHandler.error: "+eClass,eResult);
        $("#error_container .errorMsgs:first").append("<div class='e'><div class='title'>"+eClass+"</div><div class='text'>"+eResult+"</div></div>");
    },
    'exception': function(eMsg,eResult){
        console.error("eHandler.exception: "+eMsg,eResult);
        $("#error_container .errorMsgs:first").append("<div class='e'><div class='title'>"+stringify(eMsg)+"</div><div class='text'>"+stringify(eResult)+"</div></div>");
        app.nav.showContainer("error");
    }
};

// checks, if a object is ready (only valid after fired 'setReady')
// requires lodash
_.isReady = function(cObj){
    if (_.isEmpty(cObj)){
        return false;
    }
    if (_.isUndefined(cObj._ready)){
        return false;
    }
    return cObj._ready.status;
};

// preserves a ready state for each time, this function is called - so if this function is called twice, it needs to get called 'setReady' twice, to setItReady
// requires lodash
_.preserveReady = function(cObj){
    if (_.isEmpty(cObj)){
        return false;
    }

    // add ready obj
    if (_.isUndefined(cObj._ready)){
        cObj._ready = {
            status: false,
            preserve: 0,
            run: function(tType){
                ((!tType)?tType=null:null);
                var self = cObj;
                self._ready.preserve++;

                if (self._ready.preserve >= 0){
                    self._ready.status = true;
                    if (tType != null){
                        eListener.ready(tType);
                    }
                }
            }
        };
    }
    cObj._ready.preserve--;
    cObj._ready.status = false;
};

// sets the ready state (observe 'preserveReady') and calls a eListener.ready -function if 'targetType' is set or object has prop:_className
// requires lodash
_.setReady = function(cObj,targetType){
    ((!targetType)?targetType=null:null);

    if (_.isEmpty(cObj)){
        return false;
    }

    // autoFill target type
    if ((!targetType) && (!_.isUndefined(cObj._className))){
        targetType = cObj._className;
    }

    // add ready obj
    if (_.isUndefined(cObj._ready)){
        cObj._ready = {
            status: false,
            preserve: 0,
            run: function(tType){
                ((!tType)?tType=null:null);
                var self = cObj;
                self._ready.preserve++;

                if (self._ready.preserve >= 0){
                    self._ready.status = true;
                    if (tType != null){
                        eListener.ready(tType);
                    }
                }
            }
        };
    }

    // call func
    cObj._ready.run(targetType);
};

var uuid = function(tt) {
    ((!tt)?tt=32:null);
    var result = '';
    _.times(tt, function () {
        result += Math.floor(Math.random() * 16).toString(16).toUpperCase();
    });       
    return result;
};

// add a binding to use objects 'eachProp'
if (_.isUndefined(ko.bindingHandlers["eachProp"])){
    ko.bindingHandlers.eachProp = {
        transformObject: function (obj){
            var cObj = obj;
            if (!_.isUndefined(obj.data)){
                cObj = ko.utils.unwrapObservable(obj.data);
            }

            var objAssign = "value";
            if (!_.isUndefined(obj.as)){
                objAssign = obj.as;
            }

            var objAssignKey = "key";
            if (!_.isUndefined(obj.asKey)){
                objAssignKey = obj.asKey;
            }

            var properties = [];
            _.forEach(cObj,function(cValue,cKey){
                var tmp = {};
                tmp[objAssignKey] = cKey;
                tmp[objAssign] = cObj[cKey];
                properties.push(tmp);
            });
            return ko.observableArray(properties);
        },
        init: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext){
            var value = ko.utils.unwrapObservable(valueAccessor());
            var properties = ko.bindingHandlers.eachProp.transformObject(value);
            ko.bindingHandlers['foreach'].init(element, properties, allBindingsAccessor, viewModel, bindingContext);
            return { controlsDescendantBindings: true };
        },
        update: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext){           
            var value = ko.utils.unwrapObservable(valueAccessor());
            var properties = ko.bindingHandlers.eachProp.transformObject(value);
            ko.bindingHandlers['foreach'].update(element, properties, allBindingsAccessor, viewModel, bindingContext);
            return { controlsDescendantBindings: true };
        }
    };
    ko.virtualElements.allowedBindings.eachProp = true;
}
